<?php
// $Id: database_mysql.php,v 1.4 2005/03/22 01:57:21 kerkness Exp $
//*************************************************************************//
// This file is part of NoAh.
// 
// NoAh is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// NoAh is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with NoAh; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//*************************************************************************//

/**
 * Contains the NoAhDatabase class for sending commands to MySQL.
 * 
 * @package noahlib
 */


/**
 * Database class for connecting to MySQL Database, performing queries
 * and handling results.
 * 
 * @package noahlib
 */
class NoAhDatabase
{

	/**
	* @var	string
	*/
	var $host;

	/**
	* @var	string
	*/
	var $user;
	
    /**
	* @var	string
	*/
	var $password;
	
    /**
	* @var	string
	*/
	var $database;

	/**
	* @var	boolean
	*/
	var $persistent;

	/**
	* @var	string
	*/
	var $conn;

	/**
	* @var	string
	*/
	var $result;

    var $conn_name;


    /**
     * Class Constructor
     * @return void	
     */
	function NoAhDatabase()
	{   
	}

    /**
     * Opens a connection to the database or loads a connection from the connection pool.
     * 
     * NoAh saves database connections into a _REQUEST array variable known as the connection pool.
     * when openning a connection we first look to see if a connection already exists, if it does
     * we will use it, if it doesn't we will create the connection and save it into our pool.
     * 
     * 
     * NOTE: We might want to consider optionally passing forward
     * a database connection to this object.
     * 
     * @return void
     */
	function openConnection()
	{
       if ( ! $this->conn_name ) {
           $this->conn_name = ( isset($_SESSION['use_conn']) )? $_SESSION['use_conn']:'default';
       }

       // Check to see if we already have a connection to this source.
       // if we do, then we will use the exsisting connection.
       if ( $this->hasConnection( $this->conn_name ) ) {
           $this->useConnection( $this->getConnection( $this->conn_name ) );
           error_log('RE-USING OUR _REQUEST SAVED CONNECTION');
           return;
       } 
       $this->makeConnection( $this->conn_name );
       $this->saveConnection( $this->conn_name, $this->conn );
    }


   /**
    * Method accepts a different connection to use instead of
    * the existing one.
    */
    function getConnection( $cname )
    {
        return $_REQUEST['noah_connection_pool'][$cname];
    }

   /**
    * Accepts a connection to use.
    */
    function useConnection( $conn )
    {
        $this->conn = $conn;
    }

    function hasConnection( $cname )
    {
        if ( isset($_REQUEST['noah_connection_pool']) && is_array($_REQUEST['noah_connection_pool']) ) {
            foreach($_REQUEST['noah_connection_pool'] as $name=>$conn){
                if ( strtolower($name) == strtolower($cname) ) {
                    return true;
                }
            }
        }
       return false;
    }

    function saveConnection( $cname, $conn )
    {
        error_log('SAVING OUR CONNECTION TO _REQUEST');
        $_REQUEST['noah_connection_pool'][$cname] = $conn;
    }


    function makeConnection( $cname )
    {
        error_log("***** MAKING A NEW CONNECTION to $cname");
        $server = '';
        $user = '';
        $pass = '';
        $database = '';
        $prefix = '';
        $persistent = '';

        // Loop over _REQUEST[noah_conn]
        foreach($_REQUEST['noah_conn'] as $i=>$connarr ) 
        {
            if ( strtolower($connarr['name'])== strtolower($cname) ) {
                $server = (isset($connarr['host']))? $connarr['host'].':':'localhost:';
                $server .= (isset($connarr['port']))?$connarr['port']:'';
                $user = (isset($connarr['user']))?$connarr['user']:'';
                $pass = (isset($connarr['pass']))?$connarr['pass']:'';
                $database = (isset($connarr['database']))?$connarr['database']:'';
                $prefix = (isset($connarr['prefix']))?$connarr['prefix']:'';
                $persistent = (isset($connarr['persistent']))?$connarr['persistent']:'';
            }
        }

        // If no settings check _SESSION[temp_conn]
        if ( ! $server && $cname == $_SESSION['temp_conn']['name'] ) {
            $server = (isset($_SESSION['temp_conn']['host']))? $_SESSION['temp_conn']['host'].':':'localhost:';
            $server .= (isset($_SESSION['temp_conn']['port']))?$_SESSION['temp_conn']['port']:'';
            $user = (isset($_SESSION['temp_conn']['user']))?$_SESSION['temp_conn']['user']:'';
            $pass = (isset($_SESSION['temp_conn']['pass']))?$_SESSION['temp_conn']['pass']:'';
            $database = (isset($_SESSION['temp_conn']['database']))?$_SESSION['temp_conn']['database']:'';
        }

        // If no settings found trigger an error
        if ( ! $server || ! $user || ! $pass || ! $database ) {
            trigger_error("noEh: Found missing or incomplete connection settings $cname", E_USER_ERROR );
        }

        // Call the connection settings methods
        $this->setConnection( $server, $user, $pass, $database, $prefix, $persistent );
        $this->open();

     }

   /**
    * Sets the connection parameters
	* @param	string	$host	database server
	* @param	string	$user	database user
	* @param	string	$password	database user pass
	* @param	string	$database	database name
	* @param	bool	$persistent	set to true to use a persistent db connection
    * @return void
	*/
	function setConnection( $host, $user, $password, $database, $prefix = '', $persistent = false )
	{   
		$this->host = $host;
		$this->user = $user;
		$this->password = $password;
		$this->database = $database;
		$this->persistent = $persistent;
        $this->prefix = $prefix;
	}

   /**
	* Tests a database connection returns true if successful or an error if failed.
	* @param	string	$host	database server
	* @param	string	$user	database user
	* @param	string	$password	database user pass
	* @param	string	$database	database name
	* @param	bool	$persistent	set to true to use a persistent db connection
    * @return mixed
	*/
	function testConnection( $host, $user, $password, $database, $prefix = '', $persistent = false )
	{   
		$conn = mysql_connect($host, $user, $password);
		if (@!$conn){
			return "NoEh: ".mysql_error();
		}
        /* Select database */
		if ( @!mysql_select_db($database, $conn)){
			return "NoEh: ".mysql_error();;
		}
	return true;
	}

   /**
	* Opens the the database connection set {@link $conn}
    * @return boolean
	*/
	function open()
	{
        //error_log(" START MYSQL OPEN " );
		
        /* Connect to Mysql Server */
		$this->conn = mysql_connect($this->host, $this->user, $this->password);
        
        //error_log(" JUST RANT MYSQL_CONNECT :  ".$this->host.":".$this->user.":".$this->password );

		if (!$this->conn){
            error_log("$func Connecting $this->host, $this->user, $this->password :: ".mysql_error() );
			return false;
		}

        //error_log(" THIS->CONNECT :  ".$this->conn );
		
        /* Select database */
		if ( @!mysql_select_db($this->database, $this->conn)){
            error_log("Failed to load DB $this->database :: ".mysql_error() );
			return false;
		}

	return true;
	}

    /**
     * Adds {@link $prefix} to supplied name
     * @param string $name value to prefix
     * @return string
     */
    function prefix($name)
    {
        return $this->prefix.$name;
    }

   /**
	* Close the database connection 
	*/
	function close()
	{
		return(@mysql_close($this->conn));
	}
	
	/**
	* Function for returning database errors
	*/
	function error()
	{
		return (mysql_error());
	}
	
	/**
	* Function to query the database
	*/
	function query($sql)
	{
		$this->result = @mysql_query($sql, $this->conn);
		return($this->result != false);
	}
	
	/**
	* Function to get affected rows from query results
	*/
	function affectedRows()
	{
		return(@mysql_affected_rows($this->conn));
	}

	/**
	* Function to get affected rows from query results
	*/
	function insertId()
	{
		return(@mysql_insert_id($this->conn));
	}
	
	/**
	* function to return number of rows in a query results
	*/
	function numRows()
	{
		return(@mysql_num_rows($this->result));
	}

    function num_fields()
    {   
		return(@mysql_num_fields($this->result));
    }
	
	/*
	* functions to return result set
	*/
	function fetchObject()
	{
		return(@mysql_fetch_object($this->result, MYSQL_ASSOC));
	}

	/*
	* functions to return result set
	*/
	function fetchArray()
	{
		return(@mysql_fetch_array($this->result, MYSQL_ASSOC));
	}

	/*
	* functions to return result set
	*/
	function fetchRow()
	{
		return(@mysql_fetch_row($this->result));
	}

	/*
	* functions to return result set
	*/
	function fetchAssoc()
	{
		return(@mysql_fetch_assoc($this->result));
	}

	/**
	* Function to free result set
	*/
	function freeResults()
	{
		return(@mysql_free_result($this->result));
	}

    function field_name($res, $i)
    {
        return(@mysql_field_name($this->result, $i));
    }

    function fetch_field($res, $i)
    {
        return(@mysql_fetch_field($this->result, $i));
    }
    
    /**
    * @return	array	table names from our database
    */
    function listTables($dbname)
    {
     $result = mysql_list_tables($dbname);
     $i = 0;
     $tb_names = array();
     while ($i < mysql_numrows($result)) {
       $tb_names[$i] = mysql_tablename($result, $i);
       // error_log $tb_names[$i]."<br />";
       $i++;
     }
     return $tb_names;
    }

}

?>
