<?php
// $Id: NoAhContentPiece.php,v 1.4 2005/03/22 01:57:22 kerkness Exp $
   
//*************************************************************************//
// This file is part of NoAh.
// 
// NoAh is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// NoAh is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with NoAh; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//*************************************************************************//

/**
 * NoAhContentPiece.php provides methods for handling a single content Piece
 * 
 * @author Ryan Mayberry < phparch at gmail dot com >
 * @package noahlib
 */
 
/**
 * Provides methods for handling a single content Piece
 * 
 * NoAh provides the ability to build metadata about different Content Sources,
 * with each content source being made up of one or more content pieces. A 
 * NoAhContentPiece object is created for each of thoughs pieces.
 * Properties of this class can be added using NoAhObject:setAttribute();
 * 
 * @see NoAhContentSource()
 * @package noahlib
 */
class NoAhContentPiece extends NoAhObject
{

    /**
     * A user friendly version of {@link $name} for this content piece
     * @var     string   
     */
    var $label;
    /**
     * The type of input field to use when editing this content piece
     * @var     string
     */
    var $input;
    /**
     * The name (field name or xml tag) for this content piece
     * @var     string
     */
    var $name;
    /**
     * The length of this content piece
     * @var     string
     */
    var $len;
    /**
     * The order this content piece should be displayed in relatioin to other pieces
     * @var     string
     */
    var $order;
    /**
     * The default value for this content piece
     * @var     string
     */
    var $default;
    /**
     * The RDF-XML URI for this content piece
     * @var     string
     */
    var $uri;

    /**
     * An array of tables and fields that this content piece has a join with
     * @var     joins
     */
    var $joins = array();

    /**
     * URI of the of the pieces 'naoh:join' resource in source rdf file.
     * @var     string
     */
    var $join;

    /**
     * Class constructor method sets up {@link $name}
     * @param string $name name of our content piece
     * @return void
     */
    function NoAhContentPiece( $name='' )
    {
        if ( $name ) $this->name = $name;
    }

    /**
     * Returns the properties for this content piece
     * @return array
     */
    function pieceDetails()
    {
        $arr = array();
        foreach($this as $i=>$v )
        {
            $arr[$i]=$v;
        }
        return $arr;
    }
    
   /**
    * Returns an array of options based on the first value in {@link $joins}
    * @return array	
    */
    function getOptionsFromJoin()
    {
    	$options = array();
    	if ( ! $this->joins ) return $options;
    	$join = $this->joins[0];
    	
    	if( isset($join['jsource']) ){
	    	$source = $join['jsource'];
  	 	 	$mySource = new NoAhContentReader($source);
   		 	$mySource->loadSourceFile();    	
   		 	$select = "SELECT ".$join['jfield'].", ".$join['jdisplay']." 
  	 	 		FROM ".$mySource->table;
    	    $mySource->setAttribute("select_str", $select);        
   		    $results = $mySource->getContent();    
   		    foreach($results as $i=>$row){
        	  $options[$row[$join['jfield']]] = $row[$join['jdisplay']];
        	}
        } else if ( isset($join['jlist']) ) {
        	$list = new NoAhContentList($join['jlist']);
        	$values = getListValues();
   		    foreach($values as $i=>$row){
        	  $options[$row['value']] = $row['label'];
        	}
        }
      return $options;	
    }

}


?>
