<?php
// $Id: NoAhContentWriter.php,v 1.1 2005/03/22 01:59:55 kerkness Exp $
   
//*************************************************************************//
// This file is part of NoAh.
// 
// NoAh is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// NoAh is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with NoAh; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//*************************************************************************//

/**
 * Source file for NoAhContentReader Class
 *
 * NoAh provides the ability to define metadata about Content Sources, this class
 * lets you define parameters about how to update edit and add that content 
 * 
 * @author Ryan Mayberry < phparch at gmail dot com >
 * @package noahlib
 */

/**
 * Make sure we have included NoAhContentSource class
 */
require_once( CORE_PATH.'/class/noahlib/NoAhContentSource.php');

/**
 * NoAhContentReader provides methods for getting content from a NoAh Content Source.
 * 
 * This class is used in place of of normal ContentSources object specifically
 * when content needs to be retrieved.
 * 
 * @package noahlib
 */
class NoAhContentWriter extends NoAhContentSource
{

     /**
      * An array of custom properties which can be passed to {@link $fetch}
      * @var 	array
      */
     var $custom_prop = array();
     
    /**
     * An instance of a object for reading from the content source. Default is {@link MySqlReader}
     * @var     object
     */
     var $conn;

    /**
     * 
     */
     var $fields = array();

     /**
      * Accept the name of a content source and open a connection to it.
      * @param string $name name of the content source
      * @see openContentFetcher()
      * @return void
      */
    function NoAhContentWriter( $name )
    {
         $this->NoAhContentSource($name);
         // Open an connection to a content fetcher.
         $this->openContentFetcher();
         $this->conn->useTable($this->table, $this->prime);
    }

    function addField( $field, $value )
    {
        $this->conn->addField($field,$value);
    }

    function addFieldArray( $fields )
    {
        $this->conn->addFieldArray($fields);
    }

    function insert()
    {
	    $this->passPropertiesForward();
        return $this->conn->insert();
    }

    function update( $id )
    {
	    $this->passPropertiesForward();
        return $this->conn->update( $id );
    }

    function deleteItem( $id )
    {
	    $this->passPropertiesForward();
        return $this->conn->deleteItem($id);
    }
	
    /**
     * Creates an instance of {@link $fetch}
     * 
     * This method eventually choose between available 'reader classes'
     * The default being  NoAh MySQLReader.php  this class handles 
     * The physical fetching of content from the database. 
     * This would be the class file to mirror if we want to start fetcing
     * content from other datasources, then modify the following method
     * to support it.
     * 
     * @return void
     */
	function openContentFetcher()
	{
		require_once( CORE_PATH.'/class/noahlib/NoAhMySqlWriter.php');
	    $this->conn = new NoAhMySqlWriter( $this->dbconn );
	}
	
	
    /**
     * This method allow (good or bad) developers to pass thier
     * own custom properties to the {@link $fetch} object
     * 
     * @param string $name name of the custom property
     * @param string $value value of the custom property
     * @return void
     */
	function setProperty($name,$value)
	{
		$this->custom_prop[$name] = $value;
	}
	
    /**
     * Builds an array of properties found in this class.
     * @return array
     */
	function getContentProperties()
	{
	   $props = array();
       $props['custom_prop'] = $this->custom_prop;
       $props['pieces'] = $this->pieces;
       $props['source'] = $this->sourceDetails();
       return $props;
	}
	
    /**
     * Passes an array of content properties to the {@link $fetch} object
     * @see getContentProperties()
     * @see openContentFetcher()
     * @return void
     */
	function passPropertiesForward()
	{
	    $this->conn->setProperties( $this->getContentProperties() );
	}


}


?>
