<?php
// $Id: NoAhObject.php,v 1.4 2005/03/22 01:57:22 kerkness Exp $
   
//*************************************************************************//
// This file is part of NoAh.
// 
// NoAh is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// NoAh is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with NoAh; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//*************************************************************************//

/**
 * NoAhLanguage.php provides the NoAhLanguage class object which is used 
 * for managing language files.
 *
 * @author Ryan Mayberry < phparch at gmail dot com >
 * @package noahlib
 */

/**
 * NoAhLanguage contains methods for loading and language files.
 * 
 * NoAh provides a method for providing content and sharing modules
 * which support translation into multiple languages. This class makes 
 * it easy for a developer to load one or many NoAh language files
 * and access it's content in multiple translations.
 * 
 * @package noahlib
 */
class NoAhLanguage extends NoAhRap
{
   /**
    * @var  boolean a flag which can be set to true or false depending
    * on if the class should always look for multiple translations
    * or only look for one.
    */
    var $multilang = true;

   /**
    * The primary language to return or if {@link $multilang} is set
    * to false the only language which will be loaded with {@link loadTranslations}
    */
    var $mylang = 'en';

   /**
    * Array of xml data pulled from all loaded translation files
    * @var  array  
    */
    var $xdata = array();

   /**
    * An array of translated tags
    */
    var $trans = array();

    /**
     * A index of all tags which have been loaded
     * @var array   
     */
    var $tags = array();

    /**
     * An index of all languages that we have translations for
     * NOTE: There is no guarantee that each language has a complete
     * translation for each tag.
     */
    var $languages = array();

    var $lastfile;
    var $tagsbyfile = array();
    var $taglang = array();
    var $splitlang = false;

    /**
     * Class constructor method
     * @param  
     * @return void
     */
    function NoAhLanguage()
    {
    }

    /**
     * Sets {@link $mylang} 
     */
    function setLanguage($lang)
    {
        $this->myLang = $lang;
    }

    /**
     * 
     */
    function translate( $tag, $lang )
    {
        $translation = ( isset($this->trans[$tag][$lang]) ) ? $this->trans[$tag][$lang] : $tag;
        $translation = str_replace('\"','"',$translation);
        return stripslashes($translation);
    }

    function hasTranslation( $tag, $lang )
    {
        return ( isset($this->trans[$tag][$lang]) ) ? true : false;
    }

    function setTranslation( $tag, $lang, $trans )
    {
        $this->trans[$tag][strtolower($lang)] = $trans;
    }
    
    /**
     * Clears all exsisting translations
     */
    function clearTags()
    {
        $this->xdata = array();
        $this->tags = array();
        $this->languages = array();
        $this->trans = array();
    }

    /**
     * Defines a _CONSTANT for every language tag in the
     * default language of the $noUser object
     */
    function defineTags()
    {
        foreach( $this->tags as $i=>$tag )
        {
            $myconst = '_'.strtoupper($tag);
            if ( ! defined( $myconst )  ) {
                define( $myconst, $this->translate( $tag, $this->mylang ) );
            }
        }
    }

    /**
     * Loads a requested translation into {@link $xdata}
     * 
     * When calling this function $lang_file only needs to contain the
     * main 'name' of the lanugage file and doesn't need to include any
     * prefixes or the .xml extension. The method {@ling stripFileName} 
     * will remove them if they are.
     * 
     * We strip all prefixes to get the base language 
     * file name and then call {@link fineTranslations} method to look
     * for various translation files which 
     * will do that for use.
     * 
     * @param string    $lang_file  the name of the language file to load
     */
    function loadTranslation( $file )
    {
        if ( ! defined('XML_PATH') ) {
            trigger_error('noEh: XML_PATH is not defined', E_USER_WARNING );
        }

        $this->lastfile = $file;

        // NOTE: we are only loading files which are installed.
        $files = $this->findTranslations( $file );

        // Loop over each file and load it
        foreach( $files as $i => $path )
        {
            $this->setXml($path);
            $this->loadModel();
        }
        $this->xdata = $this->getData();

        $this->indexData();
        $this->defineTags();
    }

    /**
     * Loops over {@link $xdata} and sets {@link $tags} and {@link $languages}
     */
    function indexData()
    {
        foreach($this->xdata as $uri => $data )
        {
           if ( false !== strpos($uri,'lang/')) {
            $tag = str_replace('lang/','',strtolower($uri) );
            if ( ! in_array($tag,$this->tags)) {
                $this->tags[] = $tag;
                $this->tagsbyfile[$this->lastfile][] = $tag;
            }
            foreach ( $data as $lcode => $trans )
            {
                if ( ! in_array(strtolower($lcode), $this->languages )) {
                    $this->languages[] = strtolower($lcode);
                }
                $this->trans[$tag][strtolower($lcode)] = str_replace('\n',"\n",$trans);
                //$this->trans[$tag][strtolower($lcode)] = $trans;
            }
           }
          if ( false !== strpos($uri, 'translations/') ) {
             if ( isset($data['splitlang']) ) {
                 $this->splitlang = $data['splitlang'];
             }
          }
        }
    }

   /**
    * Returns and array of the full paths to all translations files
    * 
    */
    function findTranslations( $file )
    {
        $filename = $this->normalizeFileName($file);
        $files = array();

        // look for default language translation file
        if ( is_file( XML_PATH.'/lang_'.$filename ) ) {
            $files[] = XML_PATH.'/lang_'.$filename;
        }
        // 
        foreach ( $this->getIsoCodes() as $code => $langlabel )
        {
            if ( is_file( XML_PATH.'/lang_'.strtolower($code).'_'.$filename ) ) {
                $files[] = XML_PATH.'/lang_'.strtolower($code).'_'.$filename;
            }
        }
       return $files;
    }

    /**
     * This function will convert the requested file name into a properly formated
     * name of a NoAh langugage files. 
     **/
    function normalizeFileName( $filename )
    {
        // Strips any prefixes off the file name and makes sure it has
        // an .xml extension
        foreach( $this->getIsoCodes() as $code => $langlabel )
        {
          $filename = str_replace( 'lang_'.strtolower($code).'_','',$filename);
        }
        $filename = str_replace('lang_','',$filename);
        $filename = str_replace('.xml','',$filename);
      return $filename.'.xml';
    }

    function saveLanguageFile( $langfile, $lang='' )
    {
    	// Include our rdfmodeler class file.
    	require_once( CORE_PATH.'/modules/nosystem/class/rdfmodel.php' );
        $mymodel = new NoAhRdfModel();
    
        $splitlang = ( $lang ) ? 1 : 0 ;
        $uri_name = ( $lang ) ? $lang.'_'.$langfile:$langfile;
    	// DEFINE STATEMENTS FOR LANGUAGE FILE HEADER

    	// Set our Source URL
    	$luri = '../ns/translations/' . str_replace( ' ','_', $uri_name );
        $mymodel->addStatement($luri,'noah:splitlang', $splitlang );
    
    	// DEFINE STATEMENTS FOR EACH CONTENT PIECE
    	foreach($this->trans as $tag=>$trans )
    	{ 
           $uri = '../ns/lang/'. str_replace(' ','_', $tag);
           foreach ( $trans as $tlang => $tstr )
           {
               if ( $splitlang && $tlang !=  $lang ) continue;
        	   $mymodel->addStatement($uri,'noah:'.$tlang, $tstr );
           }
           unset($uri);
    	}
    
    	// SET DEFAULT STATEMENTS WHICH NEED TO BE INCLUDED IN ALL SOURCES
    
    	// Set the name of our model
    	$mymodel->setName( 'lang_'.strtolower($uri_name).'.xml' );
    
    	// Save our model
    	if ( ! $mymodel->saveModel() ){
    	     $results = "failed";
    	} else {

             $this->indexLangFiles( $langfile );
    	     $results = "success";
    	}
        return $results;
    }

    function indexLangFiles( $langfile )
    {
        $langfile = strtolower($langfile);
        include_once( CORE_PATH.'/class/noahlib/NoAhContentList.php' );
        $nlist = new NoAhContentList('noah_translations');
        if ( ! $nlist->isValue( $langfile ) ) {
             $nlist->addValue( $langfile, $langfile );
             $nlist->saveListXML();
        }
    }

    

    function getIsoCodes()
    {
        $isoc = array(
        'AA' => 'Afar',
        'AB' => 'Abkhazian',
        'AF' => 'Afrikaans',
        'AM' => 'Amharic',
        'AR' => 'Arabic',
        'AS' => 'Assamese',
        'AY' => 'Aymara',
        'AZ' => 'Azerbaijani',
        'BA' => 'Bashkir',
        'BE' => 'Byelorussian',
        'BG' => 'Bulgarian',
        'BH' => 'Bihari',
        'BI' => 'Bislama',
        'BN' => 'Bengali',
        'BO' => 'Tibetan',
        'BR' => 'Breton',
        'CA' => 'Catalan',
        'CO' => 'Corsican',
        'CS' => 'Czech',
        'CY' => 'Welsh',
        'DA' => 'Danish',
        'DE' => 'German',
        'DZ' => 'Bhutani',
        'EL' => 'Greek',
        'EN' => 'English',
        'EO' => 'Esperanto',
        'ES' => 'Spanish',
        'ET' => 'Estonian',
        'EU' => 'Basque',
        'FA' => 'Persian',
        'FI' => 'Finnish',
        'FJ' => 'Fiji',
        'FO' => 'Faeroese',
        'FR' => 'French',
        'FY' => 'Frisian',
        'GA' => 'Irish',
        'GD' => 'Gaelic', 
        'GL' => 'Galician',
        'GN' => 'Guarani',
        'GU' => 'Gujarati',
        'HA' => 'Hausa',
        'HI' => 'Hindi',
        'HR' => 'Croatian',
        'HU' => 'Hungarian',
        'HY' => 'Armenian',
        'IA' => 'Interlingua',
        'IE' => 'Interlingue',
        'IK' => 'Inupiak',
        'IN' => 'Indonesian',
        'IS' => 'Icelandic',
        'IT' => 'Italian',
        'IW' => 'Hebrew',
        'JA' => 'Japanese',
        'JI' => 'Yiddish',
        'JW' => 'Javanese',
        'KA' => 'Georgian',
        'KK' => 'Kazakh',
        'KL' => 'Greenlandic',
        'KM' => 'Cambodian',
        'KN' => 'Kannada',
        'KO' => 'Korean',
        'KS' => 'Kashmiri',
        'KU' => 'Kurdish',
        'KY' => 'Kirghiz',
        'LA' => 'Latin',
        'LN' => 'Lingala',
        'LO' => 'Laothian',
        'LT' => 'Lithuanian',
        'LV' => 'Latvian',
        'MG' => 'Malagasy',
        'MI' => 'Maori',
        'MK' => 'Macedonian',
        'ML' => 'Malayalam',
        'MN' => 'Mongolian',
        'MO' => 'Moldavian',
        'MR' => 'Marathi',
        'MS' => 'Malay',
        'MT' => 'Maltese',
        'MY' => 'Burmese',
        'NA' => 'Nauru',
        'NE' => 'Nepali',
        'NL' => 'Dutch',
        'NO' => 'Norwegian',
        'OC' => 'Occitan',
        'OM' => 'Oromo', 
        'OR' => 'Oriya',
        'PA' => 'Punjabi',
        'PL' => 'Polish',
        'PS' => 'Pashto',
        'PT' => 'Portuguese',
        'QU' => 'Quechua',
        'RM' => 'Rhaeto-Romance',
        'RN' => 'Kirundi',
        'RO' => 'Romanian',
        'RU' => 'Russian',
        'RW' => 'Kinyarwanda',
        'SA' => 'Sanskrit',
        'SD' => 'Sindhi',
        'SG' => 'Sangro',
        'SH' => 'Serbo-Croatian',
        'SI' => 'Singhalese',
        'SK' => 'Slovak',
        'SL' => 'Slovenian',
        'SM' => 'Samoan',
        'SN' => 'Shona',
        'SO' => 'Somali',
        'SQ' => 'Albanian',
        'SR' => 'Serbian',
        'SS' => 'Siswati',
        'ST' => 'Sesotho',
        'SU' => 'Sudanese',
        'SV' => 'Swedish',
        'SW' => 'Swahili',
        'TA' => 'Tamil',
        'TE' => 'Tegulu',
        'TG' => 'Tajik',
        'TH' => 'Thai',
        'TI' => 'Tigrinya',
        'TK' => 'Turkmen',
        'TL' => 'Tagalog',
        'TN' => 'Setswana',
        'TO' => 'Tonga',
        'TR' => 'Turkish',
        'TS' => 'Tsonga',
        'TT' => 'Tatar',
        'TW' => 'Twi',
        'UK' => 'Ukrainian',
        'UR' => 'Urdu',
        'UZ' => 'Uzbek',
        'VI' => 'Vietnamese',
        'VO' => 'Volapuk',
        'WO' => 'Wolof',
        'XH' => 'Xhosa',
        'YO' => 'Yoruba',
        'ZH' => 'Chinese',
        'ZU' => 'Zulu'
        );

      asort($isoc);
      return $isoc;
    }

}

?>
