<?php
// $Id: NoAhMySqlReader.php,v 1.2 2005/03/22 01:57:22 kerkness Exp $
   
//*************************************************************************//
// This file is part of NoAh.
// 
// NoAh is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// NoAh is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with NoAh; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//*************************************************************************//

/**
 * NoAhMySQLReader.php provides methods for running SELECT queries in MySQL
 *
 * NoAh provides the ability to define a set of properties about a specific 
 * Content Sources. If that content source is found ina  MYSQL database
 * then this class would be used to fetch the content. 
 *
 * @author Ryan Mayberry < phparch at gmail dot com >
 * @package noahlib
 */

/**
 * Make sure we have included our mysql database class file with NoAhDatabase
 */
require_once( CORE_PATH.'/class/db/database_mysql.php');

/**
 * NoAhMySQLReader provides methods for running SELECT queries in MySQL
 * 
 * This class can be invoked directly by any php script but if working with
 * a NoAH content source it's likely easier to use the NoAhContentReader
 * class which invoke the primary methods found here.
 * 
 * @package noahlib
 */
class NoAhMySqlReader extends NoAhDatabase
{
    /**
     * @var 	integer
     */
     var $row_count;
     /**
      * @var 	string
      */
     var $sortby;
     /**
      * @var 	string
      */
     var $sortdir;
     /**
      * @var 	integer
      */
     var $page;
     /**
      * @var 	integer
      */
     var $limit;
     
     /**
      * @var 	string
      */
     var $select_str;
     /**
      * @var 	string
      */
     var $where_str;
     /**
      * @var 	string
      */
     var $order_str;
     /**
      * @var 	string
      */
     var $limit_str;
     /**
      * @var 	array
      */
     var $pieces = array();
     /**
      * @var 	array
      */
     var $source = array();

    /**
     * @var     array
     */
     var $filter = array();

    /**
     * @var     string
     */
     var $filter_string;

     var $conn_name;

     /**
      * Class Contructor opens connection to database
      * @see openConnection()
      * @return void
      */
    function NoAhMySqlReader( $conn_name )
    {
         $this->conn_name = $conn_name;
         $this->openConnection();
    }
    
    /**
     * Sets values for properties of this class
     * 
     * This method is primarily called by NoAhContentReader to pass forward
     * properties defined for filtering and fetching content
     * 
     * @param array $props an array of name/value details of properties
     * @return void
     */
    function setProperties( $props )
    {
		foreach( $props as $name=>$value )
		{
		   //if ( $name == 'custom_prop' ) continue;
		   $this->$name = $value;
		}
		foreach($props['custom_prop'] as $name=>$value)
		{
			$this->$name = $value;
		}
    }
    
    /**
     * This function should return an array of content records that take into
     * consideration all specific parameters such as filtering, paging and
     * sorting.    
     * 
     * @return array
     */
    function getContentRecords()
    {
       $content = array();
       
       // Pass SQL query to our query method
       // If fails trigger error E_USER_ERROR
       // NOTE:   Might not always want to kill this when an error
       // happens , so consider other options like adding an error handler.
       $sql = $this->getSql();
       if( ! $res = $this->query($sql) ) {
        error_log( $this->error() );
       	trigger_error("Failed to fetch content \n$sql\n", E_USER_ERROR );
       }
       
       // Loop over results and add to $content array
       while( $row = $this->fetchArray($res) ) {
            $content[] = $this->cleanRecords($row); 
       }
       
       // return our content
       return $content;
    }

    /**
     * Cleans data after being pulled from the database, eg stripslashes()
     * 
     * Consider using the xoopstextsanitizer here. 
     * 
     * @param   array   a single row of content records
     * @return  array   cleaned content records
     */
    function cleanRecords($row)
    {
        foreach($row as $field=>$value)
        {
            $row[$field] = stripslashes($value);
        }
      return $row;
    }
    
    /**
     * Builds a complete SQL statement for fetching records from db
     * @return string
     */
    function getSql()
    {
       // Build the SQL in parts
       $select = $this->getSelect();
       $where = $this->getWhere();
       $order = $this->getOrder();
       $limit = $this->getLimit();
       
       // Combine all parts of our SQL statement to a full string.
       $sql = $select .' '.$where.' '.$order.' '.$limit;
       
       return $sql;
    }
    
    /**
     * Builds the 'SELECT' portion of our SQL statement
     * @return string
     */
    function getSelect()
    {
    	// If our select_str parameter is set we can 
    	// just return that.
    	if ( $this->select_str ) return $this->select_str; 
    	
    	$table = $this->source['table'];
    	$prime = $this->source['prime'];
    	
    	// Build the select string with what we know about our
    	// content source.
        $str = 'SELECT ';
        $n=0;
    	foreach ( $this->columns as $k=>$column )
    	{
    	    if ( $n > 0 ) $str.= ', ';
    	    $str .= $table.'.'.$column['name'];
    	$n++; }
    	
    	// NOTE: THIS IS WHERE WE NEED TO ADD SUPPORT FOR JOINS
    	// 
    	$str .= " FROM $table";
    	
    	return $str;
    }
    
    /**
     * Builds the 'WHERE' portion of our SQL statement
     * @return string
     */
    function getWhere()
    {
    	if ( $this->where_str ) return $this->where_str; 
        $str = '';
        if ( $this->filter_string ) {
            $this->filter_string = str_replace('WHERE','',$this->filter_string);
            $str = ' WHERE ';
            $str.= $this->filter_string;
            return $str;
        } elseif ( $this->filter ) {
            $str = ' WHERE ';
            for($i=0;$i<count($this->filter);$i++){
                if ( $i > 0 ) {
                    $str .= ' AND ';
                }
                $str .= $this->filter[$i]['field'];
                $str .= ' '.$this->filter[$i]['operand'].' ';
                $str .= "'".$this->filter[$i]['value']."'";
            }
        }
    	return $str;
    }
    
    /**
     * Builds the 'ORDER BY' portion of our SQL statement
     * @return string
     */
    function getOrder()
    {
    	if ( $this->order_str ) return $this->order_str; 

        if ( ! $this->sortby ) {
            $this->sortby = $this->source['prime'];
        }
        if ( ! $this->sortdir ) {
            $this->sortdir = 'DESC';
        }
    	$str = ' ORDER BY '.$this->sortby.' '.$this->sortdir;
      return $str;
    }
    
    /**
     * Builds the 'LIMIT' portion of our SQL statement
     * @return string
     */
    function getLimit()
    {
    	if ( $this->limit_str ) return $this->limit_str; 

        $startat = 0;
        if ( ! $this->limit ) {
            $this->limit = 10;
        }
        if ( $this->page && $this->page > 1 ) {
            $startat = ($this->page-1)*$this->limit;
        }
        $str = ' LIMIT '.$startat.', '.$this->limit;
        return $str;
    }
    
    /**
     * Counts the total number of content records matching {@link getSQL()}
     * but without considering {@link $limit}
     * @return integer
     */
	function getContentCount()
	{
	   $table = $this->source['table'];
	   $sql = "SELECT COUNT(*) as mycount FROM ".$this->source['table']." ";
	   $sql .= $this->getWhere();
	   
	   if ( ! $res = $this->query($sql) ) {
        error_log( $this->error() );
       	trigger_error("Failed to fetch content \n$sql\n", E_USER_ERROR );
	   }
	   $count = $this->fetchArray($res);
	  return $count['mycount']; 
	}
	
	
       // Else connect to this database connection.




}


?>
