<?php
// $Id: NoAhMySqlWriter.php,v 1.1 2005/03/22 01:59:55 kerkness Exp $
   
//*************************************************************************//
// This file is part of NoAh.
// 
// NoAh is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// NoAh is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with NoAh; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//*************************************************************************//

/**
 * NoAhMySQLWriter.php provides methods for running INSERT and UPDATE queries
 *
 * NoAh provides the ability to define metadata about content sources, if a 
 * specific content source is using a MYSQL database then this class can be
 * used to INSERT and UPDATE records in that database.
 * 
 * NOTE: I might want to consider combining NoAhDatabase, NoAhMySqlReader and 
 * NoAhMySqlWriter into a single MySqlHandler class as a single script might want 
 * to use all three classes, yet each makes their own db connection currently.
 * 
 * @author Ryan Mayberry < phparch at gmail dot com >
 * @package noahlib
 */

/**
 * Make sure we have included NoAhDatabase
 */
require_once( CORE_PATH.'/class/db/database_mysql.php');

/**
 * NoAhMySQLWriter provides methods for running INSERT and UPDATE queries
 * @package noahlib
 */
class NoAhMySqlWriter extends NoAhDatabase
{
    /**
     * @var 	array
     */
	var $fields;
    /**
     * @var 	array
     */
	// var $values;     deperciated ??
    /**
     * @var 	string
     */
	var $table;
    /**
     * @var 	string
     */
	var $prime;
    /**
     * @var 	string
     */
	var $errormsg;

    var $conn_name;
	
    /**
     * Class constructor sets {@link $table} and {@link $prime}
     * 
     * Defines the table and primary key to in our update or
     * insert query and opens a connection to our database
     * 
     * @see openConnection()
     * @param string $table name of db table to use
     * @param string $prime field name for table primary key
     * @return void
     */
    function NoAhMySqlWriter( $conn_name )
    {
    	$this->conn_name = $conn_name;
        $this->openConnection();
    }

    function useTable( $table, $prime )
    {
    	$this->table = $table;
    	$this->prime = $prime;
    }
    
    /**
     * Adds a single field/value pair to {@link $fields}
     * @param string $field name of the field 
     * @param string $value value of the field 
     * @return void
     */
    function addField($field, $value)
    {
    	$this->fields[$field] = $this->sanitize($value);	
    }
    
    /**
     * Adds an array of field/value pairs to {@link $fields}
     * @see addField()
     * @param string $fields array of field/value pairs
     * @return void
     */
    function addFieldArray($fields)
    {
    	foreach($fields as $name=>$value)
    	{
    		$this->addField($name, $value);
    	}
    }

    function clearFields()
    {
        $this->fields = array();
    }
    
    /**
     * Runs an Insert query
     * 
     * Using the values set in {@link $fields} and {@link $values} a
     * INSERT query is constructed and then run. Returns the newly
     * inserted record ID or returns false if failed.
     * 
     * @return integer/boolean
     */
    function insert()
    {
    	$sql = "INSERT INTO ".$this->table." SET ";
    	$n = 0;
    	foreach($this->fields as $name=>$value){
            if( $name == $this->prime ) continue;   // Prevents the insertion of a primary key
    		if ($n>0) $sql .= ", ";
    		$sql .= $name."='".$value."'";
    	 $n++; 
    	}
    	$sql .= "";
    	$this->sql = $sql;
        $this->clearFields();
    	if ( $this->query($sql) ) {
    		return $this->insertId();
    	} else {
    		return false;
        }
    }
    
    /**
     * Runs an Update query
     * 
     * Using the values set in {@link $fields} and {@link $values} a
     * UPDATE query is constructed and then run. Returns the number of
     * rows affected or returns false if failed.
     * 
     * @return integer/boolean
     */
    function update( $id )
    {
    	$sql = "UPDATE ".$this->table." SET ";
    	$n = 0;
    	foreach($this->fields as $name=>$value){
    		if ($n>0) $sql .= ", ";
    		$sql .= $name."='".$value."'";
    	 $n++; 
    	}
    	$sql .= " WHERE ".$this->prime." = $id";
    	$this->sql = $sql;    	
        $this->clearFields();
    	if ( $this->query($sql) ) {
    		return $this->affectedRows();
    	} else {
    		return false;
        }
    }

    /**
     * Deletes a record from the database
     * 
     * WARNING: No confirmation is provided. Only use this method
     * if you're sure you want to delete the record.
     * @param   int  $id  the id or (@link $prime} value
     * @return  boolean true if successful false if failed.
     */
    function deleteItem($id)
    {
        if ( ! $id ) return false;
        $sql = "DELETE FROM ".$this->table." WHERE ".$this->prime."=$id"; 
        $this->clearFields();
        if ( ! $this->query($sql) ) {
            $this->errormsg = $sql;
            return false;
        }
        return true;
    }
    
    /**
     * Adds slashes to value. Should be expanded to fully sanitize content
     * prior to being inserted in DB. Consider using XOOPSTEXTSANITIZER
     * 
     * @param string $value to be sanitized
     * @return string
     */
    function sanitize($value){
     $value = addslashes($value);
     return $value;
    }
	
    /**
     * Sets values for properties of this class
     * 
     * This method is primarily called by NoAhContentReader to pass forward
     * properties defined for filtering and fetching content
     * 
     * @param array $props an array of name/value details of properties
     * @return void
     */
    function setProperties( $props )
    {
		foreach( $props as $name=>$value )
		{
		   //if ( $name == 'custom_prop' ) continue;
		   $this->$name = $value;
		}
		foreach($props['custom_prop'] as $name=>$value)
		{
			$this->$name = $value;
		}
    }



}


?>
