<?php
// $Id: NoAhRap.php,v 1.4 2005/03/22 01:57:22 kerkness Exp $
   
//*************************************************************************//
// This file is part of NoAh.
// 
// NoAh is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// NoAh is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with NoAh; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//*************************************************************************//

/**
 * NoAhRap.php provides short cut methods for parsing the contents of RDF-XML files
 *
 * NoAh uses RDF-XML formatted documents to store much of the metadata it operates
 * on. NoAh also primarily uses the PHP-RAP API to handle the parsing the creation
 * of these documents. This class provides a shortcut for opening NoAh specific 
 * xml files and returning the contents.
 * 
 * For details and informaiton on RDF-RAP see the following links
 * {@link http://www.wiwiss.fu-berlin.de/suhl/bizer/rdfapi/tutorial/introductionToRAP.htm}
 * {@link http://www.wiwiss.fu-berlin.de/suhl/bizer/rdfapi/tutorial/usingthemodelapi.htm}
 *
 * @author Ryan Mayberry < phparch at gmail dot com >
 * @package noahlib
 */

/**
 * NoAhRap provides short cut methods for parsing the contents of RDF-XML files
 * 
 * NOTE: I haven't decided if this class should extend NoAhObject or 
 * the actual RDF-RAP api. Although I can't remember them all there were
 * several reasons for choosing NoAhObject instead. The main one being
 * I don't want to be TIED too tightly to RDF-RAP, should a better
 * RDF parser come along I might want to switch.
 * 
 * @package noahlib
 */
class NoAhRap extends NoAhObject
{

     /**
      * Full path to xml file we are going to parse
      * @var string       .
      */
      var $xml;
     /**
      * An array of RDF Subjects found in XML file
      * @var array
      */
      var $subjects  =   array();

     /**
      * An array of RDF Predicates found in XML file
      * @var array
      */
      var $predicates = array();

     /**
      * An array of RDF Object found in XML file
      * @var array
      */
      var $objects   =   array();

      /**
       * An array of all data found in XML file
       * @var array
       */
      var $data = array();
      
      /**
       * An instance of the RDF-RAP Memory Model
       * @var object
       */
      var $model;

   /**
    * Class constructor sets {@link $xml} 
    * @param string $xml full path to RDF-XML file
    * @return void
    */
    function NoAhRap( $xml )
    {
        $this->xml = $xml;
    }

    function setXml( $xml )
    {
        $this->xml = $xml;
    }

    /**
     * Validates {@link $xml} creates {@link $model} and parses
     * the xml document creating {@link $data} which represents
     * a multidimensional array of our XML document.
     * @return void
     */
    function loadModel()
    {
        if ( ! is_file($this->xml) ) {
            return;
        }

        // Create new MemModel
        $this->model =& new MemModel();

        // Load xml file into memModel.
        $this->model->load($this->xml);

        $ns = "http://www.streetfighter.ca/ns";
        $rdfns = "http://www.w3.org/1999/02/22-rdf-syntax-ns#";
        $it = $this->model->getStatementIterator();
        // Traverse model and get details about command
        while ($it->hasNext()) {
            $statement = $it->next();
            $sub = str_replace("$ns/",'',$statement->getLabelSubject());
            $sub = str_replace("$ns#",'',$sub);
            $sub = str_replace($rdfns.'_','',$sub);
            $sub = str_replace($rdfns,'',$sub);
            $prd = str_replace("$ns/",'',$statement->getLabelPredicate());
            $prd = str_replace("$ns#",'',$prd);
            $prd = str_replace($rdfns.'_','',$prd);
            $prd = str_replace($rdfns,'',$prd);
            $obj = str_replace("$ns/",'',$statement->getLabelObject());
            $obj = str_replace("$ns#",'',$obj);
            $obj = str_replace($rdfns.'_','',$obj);
            $obj = str_replace($rdfns,'',$obj);
            $this->data[$sub][$prd]=$obj;
        }
       
    }

    /**
     * Calls {@link loadModel()} method and returns results.
     * 
     * This would be the main method called in PHP scripts that wanted
     * to see the contents of an XML document. This method is also used
     * in other NoAh classes when they need to load an XML document.
     * 
     * @return array
     */
    function getData( )
    {
        $this->loadModel();
        return $this->data;
    }

}


?>
