<?php
// $Id: NoAhRequest.php,v 1.6 2005/03/22 01:57:22 kerkness Exp $
   
//*************************************************************************//
// This file is part of NoAh.
// 
// NoAh is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// NoAh is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with NoAh; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//*************************************************************************//

/**
 * NoAhRequest.php provides the main methods used when processing requests
 *
 * Everytime a request for a Webpage or a specific NoAh Package Command the request
 * is handled by this class. This object would be created once for all NoAh
 * requests with the primary purpose of determining the details of the specific
 * request, understanding how to handle the request and how to return the results
 * while also providing methods for a developer to control this process.
 *
 * @author Ryan Mayberry < phparch at gmail dot com >
 * @package noahlib
 */

 /**
 * @package noahlib
 */
class NoAhRequestHandler
{
    var $request = array();
    var $pkgxml = array();
    var $cmdchain = array();
    var $alias = array();
    var $output = array();
    var $header = array();
    var $noUser;
    var $noLang;
    
    /**
     * Accepts the first mod,pkg,cmd request.
     */
    function NoAhRequestHandler( $mod, $pkg, $cmd )
    {
        $this->noUser =& new NoAhUser();
        $this->noLang =& new NoAhLanguage();
        $this->cmdchain['main'] = md5(strtolower($mod.$pkg.$cmd));
        $this->request['main'] =& new NoAhRequest( $mod, $pkg, $cmd );
        $this->request['main']->setUserObject( &$this->noUser );
        $this->pkgxml[$pkg] = $this->request['main']->loadPkgCmdModel();
        $this->request['main']->loadRequestProperties();
        $this->findIncludes('main');
    }
    
    /**
     * Adds a new request to our chain of commands.
     */
    function addRequest( $name, $mod, $pkg, $cmd )
    {	
        // *********** DEBUG BLOCk
        error_log("* ADDING REQUEST *  $name $mod $pkg $cmd");
        // *********** DEBUG BLOCk

        // Don't duplicate names, accept the first occurance
        if ( isset($this->request[$name]) ) {
            return;
        }

        // Check to see if the same command is being requested more than once.
        // If so, we don't want to duplicate the command, but duplicate the
        // the output.
        foreach($this->cmdchain as $chainname=>$chaincmd ){
          if ( md5(strtolower($mod.$pkg.$cmd)) == $chaincmd ) {
              $this->alias[$chainname][] = $name;
              return;
          }
        }

        $this->request[$name] =& new NoAhRequest( $mod, $pkg, $cmd ); 
        if ( ! isset( $this->pkgxml[$pkg] ) ) {
            error_log("LOADING PACKAGE COMMAND MODEL FOR $pkg $cmd");
            $this->pkgxml[$pkg] = $this->request[$name]->loadPkgCmdModel();
        } else {
            error_log("SET THE PACKAGE COMMAND MODEL PREVIOUSLY LOADED - SET FOR $name $pkg");
            $this->setPkgCmdModel( $name, $pkg );
        }
        $this->request[$name]->loadRequestProperties();
        $this->findIncludes($name);
        return true;
    }


    function setPkgCmdModel( $name, $pkg )
    {
       $this->request[$name]->setData( $this->pkgxml[$pkg] );
    }

    function getPrimaryRequestHeader()
    {
       $args =  $this->request['main']->getCmdProp( $this->request['main']->cmd );
       $header['page_title'] = ( isset($args['title']) )? $args['title'] : '';
       $header['page_desc'] = ( isset($args['desc']) )? $args['desc'] : '';
       $header['page_name'] = ( isset($args['name']) )? $args['name'] : '';
       $header['page_label'] = ( isset($args['label']) )? $args['label'] : '';
      return $header;
    }

    function findIncludes($name)
    {           
        error_log("LOOKING FOR INCLUDES IN $name");
        if ( ! $this->request[$name]->includes ) {
            return false;
        }
        foreach( $this->request[$name]->includes as $i=>$include )
        {
            //error_log("CONSIDERING INCLUDE $i FOR $name");
            //foreach( $include as $ii=>$vv ) {
            //error_log(time()."INCLUDE DETAILS IN $i FOR $name --- $ii == $vv ");
            //}
            $this->addRequest( $include['name'],$include['mod'],$include['pkg'],$include['cmd']);
        }
    }

    function processRequests()
    {   
        // Set default language to whatever is in the user record
        $this->noLang->setLanguage( $this->noUser->getLang() );

        //print '<pre>';
        foreach( $this->request as $name=>$object )
        {
            error_log("** PROCESSING REQUEST  $name");
            $this->processRequest( $name );
        }
       // print '</pre>';
    }

    function processRequest( $proc_req_name )
    {   
        // Create references to global NoAh objects
        $noUser =& $this->noUser;
        $noLang =& $this->noLang;
        $noAh =& $this->request[$proc_req_name];
        
        // Load translations that come with this request
        foreach( $noAh->languages as $i=>$langtag )
        {   
            $noLang->loadTranslation($langtag);
        }

        // 
        foreach($this->getPrimaryRequestHeader() as $hn => $hv )
        {
            $noAh->addContent($hn, $hv);
        }

        // LOG THIS REQUEST
        //error_log("REQUEST $mod : $pkg : $cmd ");
        $mod_path = CORE_PATH.'/modules/'.$noAh->mod;
        // Include the main module script.
        include( $mod_path.'/'.$noAh->mod.'_index.php' );
    
        // Get array of page scripts.
        $scripts = $noAh->getScript();
    
        // Loop over all scripts and run them one at a time.
        for ($i=0;$i<count($scripts);$i++) {
            $incfile = htmlentities($scripts[$i]);
            if ( ! $incfile || ! is_file($incfile) ) continue; 
                include($incfile);
        }
    
       // Check to see if a PAGE_THEME is required for this request.
       //if ( defined('PAGE_THEME')) {
       //    $noAh->addTheme( PAGE_THEME );
       //}

       $this->output[$proc_req_name] = $noAh->returnResults();
       // Return results.
       // $noAh->returnResults();
    }

    function processResults()
    {
        $results = ( isset($this->output['theme']) )? $this->output['theme'] : $this->output['main'];
        
        error_log("SET MAIN OUTPUT STRING");

        // Loop over all output and insert into our results.
        foreach ($this->output as $name=>$content)
        {
            error_log("REPLACE INCLUDED CONTENT $name ");
            $results = str_replace( '[noinclude:'.$name.']', $content, $results );
        }

       echo $results;
    }
      
}

/**
 * NoAhRequest handles all requests sent to a NoAh module or application
 * @package noahlib
 */
class NoAhRequest extends NoAhRap
{

    /**
     * An array of scripts assigned to current request
     * @var     array       
     */
    var $scripts = array();

    /**
     * An array of additional mod/pkg/cmd requests that will be performed
     * along with this one.
     * @var     array       
     */
    var $includes = array();

    /**
     * An array of functions, or commands that can be passed to scripts.
     * @var     array       
     */
    var $func = array();

    /**
     * An array of template files to use when returning this request.
     * @var     array       
     */
    var $templates = array();

    /**
     * An array of content that can be returned with the request
     * @var     array       
     */
    var $content = array();

    /**
     * The name of file to use as a theme for returning request.
     * @var     theme       
     */
    var $theme;

    /**
     * Javascript files that can be loaded with templates.
     * @var     array       
     */
    var $js = array();

    /**
     * Name of the module the request is being made for.
     * @var     string       
     */
    var $mod;
    var $mod_path;

    /**
     * Name of the module package the request is being made for.
     * @var     string       
     */
    var $pkg;

    /**
     * Name of the command being placed.
     * @var     string       
     */
    var $cmd;

    /**
     * Name of the content source file associated with {@link $cmd}
     * @var     string       
     */
    var $source_file;

    /**
     * An array of Content source details for the sources found in {@link $source_file}
     * @var     array       
     */
    var $sources = array();
    var $languages = array();

    /**
     * An instance of the RDF-RAP Memory Model
     * @var     object       
     */
    var $pmodel;

    /**
     * An array of connection handlers
     * @var     array
     */
    var $connpool = array();

    var $noUser;
    var $noLang;
    /**
     * Class Constructor sets {@link $mod} {@link $pkg} and {@link $cmd}
     * 
     * The {@link $mod} {@link $pkg} and {@link $cmd} set here are the primary
     * controllers for how this request is handled. The constructor will take
     * these variables load the XML file specific for this {@link $pkg} and fetch
     * details about the {@link $cmd}. It will then proceed to check authorization
     * for running the command and look for attached content sources.
     * 
     * @see loadPkgCmdModel()
     * @see getCmdProp()
     * @see findCommandSources()
     * @param string $mod name of the requested module
     * @param string $pkg name of the requested package
     * @param string $cmd name of the requested command
     * @return void
     */
    function NoAhRequest( $mod, $pkg, $cmd )
    {
        $this->mod = $mod;
        $this->mod_path = CORE_PATH.'/modules/'.$this->mod;
        $this->pkg = $pkg;
        $this->cmd = $cmd;
    }

    function setUserObject( $obj )
    {
        $this->noUser =& $obj;
    }

    function loadRequestProperties()
    {
        $cmdarr = $this->getCmdProp( $this->cmd );

        // Consider permissions for executing this
        // command
        if (isset($cmdarr['usrlvl']) && $cmdarr['usrlvl']) {
            $this->checkAuth($cmdarr['usrlvl']);
        }

        // Assign var's with cmdarr details.
        if (isset($cmdarr['script'])) {
            $this->scripts[] = $this->mod_path.'/scripts/'.$cmdarr['script'];
        }
        
        if (isset($cmdarr['template'])) {
            $this->templates[] = $cmdarr['template'];
        }
        
        if (isset($cmdarr['func'])) {
            $this->func[] = $cmdarr['func'];
        }
        
        if (isset($cmdarr['theme']) && $cmdarr['theme'] ) {
            $themearr = explode('/',$cmdarr['theme']);
            $this->addInclude('theme',$themearr[0],$themearr[1],$themearr[2]);
        }

        // Checks for assigned sources in page_[COMMAND].xml 
        $this->findCommandSources();
        
    }

    /**
     * Function sets XML data manually instead of processing the pkgxml file.
     * 
     * The main NoAhRequestHandler will call this method instead of loadPkgCmdModel
     * if a command from this same package is already in our chain.
     */
    function setData( $pkgxml )
    {
        $this->data = $pkgxml;
    }

    /**
     * Finds the Package XML file and creates object {@link $pmodel} which
     * is an instance of the RDF-API
     * 
     * @param string $param1 name to declare
     * @param string $param2 value of the name
     * @return integer
     */
    function loadPkgCmdModel()
    {
        /**
         * NOTE:ANOTHER IDEA, one way to quickly improve this (but I dont have time for it
         * now) is to load $mod and $pkg details from their own RDF files, something
         * that would let you create more dynamic naming conventions or locations of
         * these core definition files. But granted with the overhead of having to read
         * one more file to get this info.
         */
        $xmlfile = 'pkg_'.$this->pkg.'.xml';

        if ( is_file(CORE_PATH.'/xml/'.$xmlfile ) ) {
            $xml = CORE_PATH.'/xml/'.$xmlfile;
        }else if(is_file(XML_PATH.'/'.$xmlfile )){
            $xml = XML_PATH.'/'.$xmlfile;
        }

        // Call the constructor function of NoAhRap.
        $this->NoAhRap($xml);
        $this->loadModel();
        // Create new MemModel
        //$this->pmodel =& new MemModel();

        // Load xml file into memModel.
        //$this->pmodel->load($xml);
        return $this->data;
    }

    /**
     * Accepts a single command name and returns an array of command properties
     * 
     * This uses {@link $pmodel} methods to build an array of properties
     * that have been declared about our command.
     * 
     * @see loadPkgCmdModel()
     * 
     * @param string $cmd the name of the command to look for
     * @return array
     */
     function getCmdProp($cmd)
     {
        $cmdarr = array();
        foreach($this->data as $uri=>$data)
        {
            if ( strtolower($uri) == strtolower('cmd/'.$cmd) ) {
               $cmdarr = $data;
            }
        }
        return $cmdarr;
     }
    


    /**
     * Checks the authority level of a User.
     * 
     * If no user level is present or the set user level is
     * less than the required user level. Kill the request here.
     * 
     * NOTE: This method is not intended to provide all the security
     * necessary to run a command. What it does provide is the ability
     * to halt the execution of a command based on a NoAh specific
     * flag which can be set by a whichever script or process executed
     * this request.. 
     * 
     * Validating that the user is who they say there are has been 
     * left out on purpose. I view that feature if necessary should be 
     * provided by a larger architecture design. Of which NoAh should be
     * a component of and not the primary controller. It could be that 
     * controller which sets the flag we check here.
     * 
     * @param int $lvl a user level to check against
     * @return void
     */
    function checkAuth($lvl){
       
        if ( ! $this->noUser->isGroupMember($lvl) ) {
            $msg = "PERMISSION DENIED ON ".$this->cmd;
            echo($msg);
            trigger_error($msg, E_USER_ERROR );
        }
        
      //  if ( ! isset($_SESSION['noUser']) || ! $_SESSION['noUser'] 
      //      || $_SESSION['noUser'] < $lvl ){
      // }
    }


    // ALTERNATIVE getCmdProp METHOD BELOW. FOLLOWING IS MY FIRST APPROACH
    // TO LOOKING UP DATA IN AN RDF MODEL. THIS APPOACH IS GOOD EXCPET I SUCK
    // AT REGEX AND THUS I COULDN'T GET THE  $this->pmodel->findRegex() TO WORK
    // AND ALSO BE CASE INSENSITIVE.
    //
    // IF I EVER BECOME OR FIND A REGEX EXPERT THEN WE SHOULD SWITCH TO THE 
    // FOLLOWING APPROACH.
    /**/
    function oldgetCmdProp( $cmd )
    {
        // Define var cmdarr
        $cmdarr = array();
        
        // Assign the base NoAh namespace to var $ns
        $ns = "http://www.streetfighter.ca/ns";
        
        // Search our model for requested command.
        $cmduri = "$ns/cmd/".$cmd;
        $res = $this->pmodel->find( new Resource( $cmduri ), NULL, NULL );
        
        // Get Iterator from model
        $it = $res->getStatementIterator();
        // Traverse model and get details about command
        while ($it->hasNext()) {
        	$statement = $it->next();
            $prd = str_replace("$ns#",'',$statement->getLabelPredicate());
        	$cmdarr[$prd]=$statement->getLabelObject();
        }

        if ( ! $cmdarr ) {
            $this->_noEh("Failed to find command $cmduri", E_USER_ERROR );
        }

        // DEBUG LOOP
        //error_log("NoAhRequest.php getCmdProp DEBUG");
        //foreach( $cmdarr as $k=>$v){
        //    error_log("$k = $v");
        //}

        return $cmdarr;
    }
     //*/ 

    /**
     * Adds a command to our request object
     * 
     * A NoAh request can handle more than one command. Commands
     * can be loaded using this method. The method assumes the 
     * command belongs to {@link $pkg} and {@link $cmd} and will set
     * additional values to {@link $scripts} and {@link $templates}
     * 
     * @param string $cmd name of the command to add
     * @return void
     */
    function addCommand( $cmd )
    {
        $cmdarr = $this->getCmdProp( $cmd );
        // Assign var's with cmdarr details.
        $this->scripts[] = $cmdarr['script'];
        $this->templates[] = $cmdarr['template'];
    }

    function addInclude( $name, $mod, $pkg, $cmd )
    {
        $n = count($this->includes);
        $this->includes[$n]['name'] = $name;
        $this->includes[$n]['mod'] = $mod;
        $this->includes[$n]['pkg'] = $pkg;
        $this->includes[$n]['cmd'] = $cmd;
    }

    /**
     * Returns an array of {@link $templates}
     * @return array
     */
    function getTemplates()
    {
       return $this->templates;
    }

    /**
     * Adds a template name to {@link $templates}
     * @param string $template name of the template to add
     * @return void
     */
    function addTemplate( $template )
    {
       $this->templates[] = $template;
    }
    /**
     * Changes the value of an intem in {@link $templates}
     * @param string $index the array index of the template to change
     * @param string $template the new template name to use
     * @return void
     */
    function changeTemplate( $index, $template )
    {
       $this->templates[$index] = $template;
    }

    /**
     * Adds a script name to {@link $scripts}
     * @param string $script name of the script to add
     * @return void
     */
    function addScript( $script )
    {
        $this->scripts[] = $script;
    }

    /**
     * Checks to see if any values have been added to {@link $scripts}
     * @return boolean
     */
    function hasScript()
    {
        return ( count($this->scripts) )? true : false;
    }
    
    /**
     * Returns array of values in {@link $scripts}
     * @return array
     */
    function getScript()
    {
        return $this->scripts;
    }

    /**
     * Adds name of a javascript file to {@link $js}
     * @param string $js name of the javascript file
     * @return void
     */
    function addJs( $js )
    {
        $this->js[] = $js;
    }

    /**
     * Checks to see if there are any values in {@link $js}
     * @return boolean
     */
    function hasJs()
    {
        return ( count($this->js) )? true : false;
    }
    
    /**
     * Returns array of javascript files in {@link $js}
     * @return array
     */
    function getJs()
    {
        return $this->js;
    }

    /**
     * Adds a function name to {@link $func}
     * @param string $func name of the function to add
     * @return void
     */
    function addFunc( $func )
    {
        $this->func[] = $func;
    }

    /**
     * Checks to see if there are any values in {@link $func}
     * @return boolean
     */
    function hasFunc()
    {
        return ( count($this->func) )? true : false;
    }
    
    /**
     * Returns array of functions in {@link $func}
     * @return array
     */
    function getFunc()
    {
        return $this->func;
    }

    /**
     * Returns the value of  {@link $source_file}
     * @return string
     */
	function getSourceFile()
	{
		return $this->source_file;
	}

    /**
     * Checks to see if there are any value in {@link $lang}
     * @return boolean
     */
    function hasLang()
    {
        return ( $this->languages )? true : false;
    }
    
    /**
     * Returns the value of  {@link $theme}
     * @return string
     */
    function getTheme()
    {
    //    return $this->theme;
    }

    /**
     * Sets the value of  {@link $theme}
     * @param string $theme name of the theme to add
     * @return void
     */
    function addTheme( $theme )
    {
    //    $this->theme = $theme;
    }

    /**
     * Checks for a value in  {@link $theme}
     * @return boolean
     */
    function hasTheme()
    {
     //  return ( $this->theme )? true : false;
    }


    /**
     * Returns the value of  {@link $content}
     * 
     * The convention when using NoAh is to add all content
     * that needs to be displayed as output to {@link $content}
     * this method returns everything that has been added.
     * 
     * This method is called by the template process
     * which allows content to be handed over to a seperate 
     * template system, but it can also be called by scripts
     * which wish to manipulate content before display.
     * 
     * @see addContent()
     * @see clearContent()
     * @return array
     */
    function getContent()
    {
        return $this->content;
    }
    
    /**
     * Adds values to {@link $content}
     * 
     * The convention when using NoAh is to add all content
     * that needs to be displayed as output to {@link $content}.
     * 
     * @param string $name name to use as the content variable in template
     * @param string $content actual content to be added.
     * @return void
     */
    function addContent( $name, $content )
    {
        $this->content[$name] = $content;
    }
    
    /**
     * Clears all values in {@link $content}
     * @return void
     */
    function clearContent()
    {
        $this->content = array();
    }

    /**
     * returnResutls tells the request object to output it's content and templates
     * 
     * This method loops over all {@link $content} and converts 
     * everything to local variables.
     * 
     * Output is initiated by including our first value from {@link templates}
     * it is the responsibility of the first template to include any
     * subsequent templates.
     * 
     * If a {@link $theme} variable has been set then NoAh will 
     * automatically set this as the first value in {@link $templates}. This
     * let's us declare a main theme that can handle multiple included templates
     * 
     * This would typically be the last method called when executing
     * a NoAh request.
     * 
     * @param string $param2 value of the name
     * @return void
     */
    function returnResults()
    {
        // If our defined template is 'returnResults' then
        // we will simply output the value of $this->content[results];
        if ( $this->templates[0] == 'returnResults' ) {
            echo( $this->content['results'] );
            return;
        }
        $results = '';

        // build a $tpl array of full paths to our template files.
        $tpls = array();
        for($i=0,$n=0;$i<count($this->templates);$i++,$n++)
        {
            $tpls[$n] = $this->mod_path.'/templates/'.trim($this->templates[$i]);
        }
        
        //if ( $this->hasTheme() ) {
        //    $tpls[0] = $this->mod_path.'/templates/'.trim($this->getTheme());
        //}
        // NOTE: We only include our first template
        // Although our we could have included many with
        // our request. The first template is always responsible
        // for including any additional templates.
        if ( $tpls && is_file( $tpls[0] ) ) {
            // roll everything that has been added to our
            // content array as variable, value combinations.
            if ( $this->content && is_array($this->content) ) {
              foreach ( $this->content as $name=>$v ){
                $$name = $v;
              }
            }
            // INclude first template.
            ob_start();
            include($tpls[0]);
            $results = ob_get_contents();
            ob_end_clean();
            
        } else {    // No template provided, or template wasn't a valid file.
            $this->noEh("I didn't find template for command ".$this->cmd, E_USER_ERROR );
        }
       return $results;
    }
    
    /** 
     * Searches for a sources file that might define content sources
     * that should be loaded with this command. Sets {@link $source_file}
     * 
     * @see loadCommandSources()
     * @see processCommandSources()
     * @return void
     */
    function findCommandSources(){
        $xmlfile = 'page_'.strtolower($this->cmd).'.xml';
        $path = '';
        if ( is_file(CORE_PATH.'/xml/'.$xmlfile ) ) {
            $this->source_file = CORE_PATH.'/xml/'.$xmlfile;
        }else if(is_file(XML_PATH.'/'.$xmlfile )){
            $this->source_file = XML_PATH.'/'.$xmlfile;
        }

        // If we have a source file, find all the content sources
        // linked to this command and build 
        if ( $this->source_file ) {
            $this->loadCommandSources();
            $this->processCommandSources();
        }
    }

    /**
     * Loads the file {@link $source_file} and sets {@link $sources}
     * 
     * @see findCommandSources()
     * @return void
     */
    function loadCommandSources()
    {
    	$sRap = new NoAhRap( $this->source_file );  
        $sources = $sRap->getData();
        foreach( $sources as $uri=>$data)
        {
            if ( substr($uri,0,7)=='source/') {
                $this->sources[] = $data;
            }
            if ( substr($uri,0,8)=='include/') {
                $this->includes[] = $data;
            }
            if ( substr($uri,0,12)=='translation/') {
                $this->languages[] = $data['name'];
            }
        }
    }

    /**
     * Consideres the values in {@link $sources} and loads content
     * if required. Content is added to {@link $content}
     * 
     * @see addContent()
     * @see loadCommandSources()
     * @return void
     */
    function processCommandSources()
    {
        require_once(CORE_PATH.'/class/noahlib/NoAhContentReader.php');
        foreach($this->sources as $k=>$source)
        {
            $mySource = new NoAhContentReader($source['source']);
            $mySource->loadSourceFile();
            foreach($mySource->sourceDetails() as $n=>$v){
                $this->sources[$k][$n] = $v;
            }
            $this->sources[$k]['pieces'] = $mySource->getPieces();
            if ( isset($source['fetch']) && $source['fetch'] ) {
            $this->sources[$k]['content'] = $mySource->getContent();
            $this->sources[$k]['columns'] = $mySource->getColumns();
            $this->sources[$k]['name'] = $source['name'];
            unset($mySource);
           }
           if ( isset($source['render']) && $source['render'] ) {
               $arr['content'] = $this->sources[$k]['content'];
               $arr['columns'] = $this->sources[$k]['columns'];
               $this->addContent($source['name'],$arr);
           }
        }

    }
		
    /**
     * Triggers a NoAh Error 
     *  
     * @param string $msg Error message to use
     * @param string $type type of user error eg: E_USER_ERROR
     * @return void
     */
	function noEh($msg,$type)
	{
        if ( $type == 256) {
          echo("noEh: $msg, err:$type");
          die;
        }else{
          error_log($msg, $type);
        }
	}

    /**
     * depreciated
     * @see noEh()
     */
    function _noEh($msg, $type)
    {
        $this->noEh($msg,$type);
    }

}


?>
