<?php
// $Id: NoAhObject.php,v 1.4 2005/03/22 01:57:22 kerkness Exp $
   
//*************************************************************************//
// This file is part of NoAh.
// 
// NoAh is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// NoAh is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with NoAh; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//*************************************************************************//

/**
 * NoAhUser.php provides the NoAhUser class object which is used for user authorization
 *
 * @author Ryan Mayberry < phparch at gmail dot com >
 * @package noahlib
 */

/**
 * NoAhUser contains methods for validating and managing web site users.
 * 
 * This class is inteded to provide a single method of validating users
 * across different user authentication models. This class does not provide
 * any methods for authorizing users. User Authorization should take place
 * within a specific module or other process, this is just responsible for
 * verifying a user.
 * 
 * 
 * @package noahlib
 */
class NoAhUser
{
   /**
    * Creates an instance of a User verification handler class.
    * @var  object  
    */
    var $handler;

    /**
     * Class constructor method
     * @return void
     */
    function NoAhUser( $handler = 'NoUsersVerify' )
    {
        $this->handler =& new $handler;
    }

    function isUser()
    {
        return $this->handler->checkIsUser();
    }

    function isAdmin()
    {
        return $this->handler->checkIsAdmin();
    }

    function username()
    {
        return $this->handler->getUserName();
    }
    function uid()
    {
        return $this->handler->getUserId();
    }

    function isGroupMember( $gid=3 )
    {
        return $this->handler->checkIsGroupMember( $gid );
    }

    function getLang()
    {
        return $this->handler->getLang();
    }

    function deny()
    {
        header("Location: ".SITE_URL."/page.php?op=noEh403");
    }

}


/**
 * NoUserAuthorize provides Authorization for NoAh NoUsers module.
 * 
 * @package noahlib
 */
class NoUsersVerify
{
    /**
     * Class constructor method
     * @return void
     */
    function NoUsersVerify()
    {
    }

    /**
     * Checks to see if _SESSION belongs to a actual NoAh User
     */
    function checkIsUser()
    {
        if ( isset($_SESSION['noUser']) && isset($_SESSION['noUserGroups']) ){
          if( $_SESSION['noUser'] && $_SESSION['noUserGroups'] ) {
            return 1;
          }
        }
       return 0;
    }
    
    /**
     * Checks to see if _SESSION belongs to a actual NoAh Admin User
     */
    function checkIsAdmin()
    {
        if ( ! $this->checkIsUser() ) {
            return 0;
        }
        if ( in_array(1, $_SESSION['noUserGroups']) ) {
            return 1;
        }
      return 0;
    }


   /**
    * Checks to see if the current user is part of any specific
    * user group.
    */
    function checkIsGroupMember( $gid=3 )
    {
        if ( ! $this->checkIsUser() ) {
            return 0;
        }
        if ( in_array( $gid, $_SESSION['noUserGroups']) ) {
            return 1;
        }
       return 0;
    }

    function getUserName()
    {
        if ( ! $this->checkIsUser() ) {
            return 0;
        }
        return $_SESSION['noUserName'];
    }
    
    function getUserId()
    {
        if ( ! $this->checkIsUser() ) {
            return 0;
        }
        return $_SESSION['noUserID'];
    }

    /**
     * Checks to see if the user has specified a specific
     *  language for a preference.
     */
    function getLang()
    {
       $userLang = ( isset($_SESSION['noUserLang']) ) ? $_SESSION['noUserLang'] : 'en';
       return $userLang;
    }

}


/**
 * XoopsUserAuthorize provides Authorization for XOOPS based websites.
 * 
 * This class is used when running NoAh as a xoops Module.
 * 
 * @package noahlib
 */
class XoopsUsersVerify
{
    /**
     * Class constructor method
     * @return void
     */
    function XoopsUsersVerify()
    {
    }

    /**
     * Checks to see if _SESSION belongs to a actual NoAh User
     */
    function checkIsUser()
    {
    }
    
    /**
     * Checks to see if _SESSION belongs to a actual NoAh Admin User
     */
    function checkIsAdmin()
    {
    }

   /**
    * Checks to see if the current user is part of any specific
    * user group.
    */
    function checkIsGroupMember( $groupid=3 )
    {
    }
    
    /**
     * Checks to see if the user has specified a specific
     *  language for a preference.
     */
    function getLang()
    {
    }

}



?>
