<?php
// $Id: NoAhValidator.php,v 1.1 2005/03/22 01:59:55 kerkness Exp $
   
//*************************************************************************//
// This file is part of NoAh.
// 
// NoAh is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// NoAh is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with NoAh; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//*************************************************************************//

/**
 * NoAhValidator.php provides methods for defining validation requirement
 *
 * @author Ryan Mayberry < phparch at gmail dot com >
 * @package noahlib
 */

/**
 * NoAhValidator methods for defining and conducting validation.
 * 
 * This class is a work in progress. Validation requirements are
 * defined as a series of {@link $conditions}, each condition expressing
 * a 'field' 'operand' and 'value'.
 * @package noahlib
 */
class NoAhValidator extends NoAhObject
{
    /**
     * An instance of {@link NoAhContentSource}
     * @var object
     */
	var $source;
    /**
     * An array of field/value pairs to validate 
     * @var array
     */
	var $post;
    /**
     * An array of conditions to consider. 
     * @var array
     */
	var $conditions = array();
    /**
     * An array of errors which can be retrieved after validation
     * @var array
     */
	var $errors;
    /**
     * An array of field which should be {@link $post} can be validated against
     * @var array
     */
	var $field_index = array();

    /**
     * Class constructor method
     * @see indexSourceFields()
     * @param object $source a NoAhContentSource object to validate
     * @return void
     */
    function NoAhValidator( $source='' )
    {
    	$this->source = $source;
    	if ( $this->source ){
	    	$this->indexSourceFields();
	    }    	
    }
    
    /**
     * Returns an array of values from {@link $errors}
     * @return array
     */
    function getErrors()
    {
    	return $this->errors;
    }
    
    /**
     * Accepts and array of data to validate and returns all
     * validated fields.
     * 
     * This method considers {@link $field_index} and strips
     * any submitted variables which are not found in 
     * {@link $field_index}. This allows us to specifically 
     * declare which fields should be validated.
     * 
     * By default if we have ran {@link indexSourceFields()} all
     * pieces from our content source will validated
     * 
     * @param array $post an array of values to validate
     * @return array
     */
    function validate( $post )
    {
    	// all variables in post, must be part of our
    	// field index if we have one
    	if ( $this->field_index ){
    		foreach($post as $name => $value ){
    			if ( ! in_array($name,$this->field_index) ) {
    				unset($post[$name]);
    			}
    		}
    	}
    
        // HAHA, the snooper will see no validation actually occurs yet.
		foreach( $this->conditions as $i=>$rule )
		{
			if ( isset($post[$rule['field']]) ){
				//$this->validateField($post['']);
			}
		}
	  return $post;	
    }
    
    /**
     * Method for validating a single field : In Construction
     */
    function validateField($field, $condition)
    {
    }

    /**
     * Adds a set of values to {@link $conditions}
     * @param string $field to validate
     * @param string $operand to use in validation eg: ==,!=,<,>
     * @param string $value value to validate against
     * @return void
     */
    function addCondition( $field, $operand, $value )
    {
    	$n = count($this->conditions);
    	$this->conditions[$n]['field'] = $field;
    	$this->conditions[$n]['operand'] = $operand;
    	$this->conditions[$n]['value'] = $value;
    }
    
    /**
     * Adds all content pieces from {@link $source} to {@link $field_index}
     * @return void 
     */
    function indexSourceFields()
    {
    	$pieces = $this->source->getPieces();
    	foreach($pieces as $k=>$piece){
    		$this->field_index[$k] = $piece->name;
    		
    		// Auto Add Conditions Here.
    		if ( $piece->input == 'datetime' ){
    			$this->addCondition($piece->name, 'format', 'strtotime');
    		}
    		
    		
    	}

    }
    
    
    /**
     *
     */
    function checkFormat()
    {
    }
    


}


?>
