<?php

// $Id: NoAhWebPage.php,v 1.2 2005/03/22 01:57:22 kerkness Exp $
   
//*************************************************************************//
// This file is part of NoAh.
// 
// NoAh is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// NoAh is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with NoAh; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//*************************************************************************//

/**
 * NoAhWebPage.php provides methods for handling a NoAh defined Web Page
 *
 * NoAh provides the ability to define a set of properties about a specific 
 * web page. This includes what script that page should run, what content 
 * that page should display and what template that page should use when returning
 * content to the user.
 *
 * @author Ryan Mayberry < phparch at gmail dot com >
 * @package noahlib
 */
 
/**
 * NoAhWebPage provides methods for handling a NoAh defined Web Page
 * 
 * Properties of this class would typically be populated from the metadata
 * known about the specific requested CMD. 
 * 
 * Properties are added to this class by calling NoAhObject:setAttribute() 
 * method.
 * 
 * @package noahlib
 */
class NoAhWebPage extends NoAhObject
{
  /**
   * Name of our Web Page. This will always match a {@link NoAhRequest::$cmd}
   * @var string
   */
   var $name;
   /**
    * Name of the package that contains the page {@link $name}
    * @var string
    */
   var $package;
   
   /**
    * Name of the package that contains the page {@link $name}
    * @var string
    */
   var $mod;
   
   /**
    * The label or user friendly version of [@link $name}
    * @var string
    */
   var $label;
   /**
    * A variable which can be used as the title of our page
    * @var string
    */
   var $title;
   /**
    * An array of keywords which can be included in a template header
    * @var array
    */
   var $keywords;
   /**
    * A theme which our page can use as a wrapper
    * @var string
    */
   var $theme;
   var $is_theme;
   /**
    * A description of this page
    * @var string
    */
   var $desc;
   /**
    * The XML file name which contains linked Content Sources
    * @var string
    */
   var $source_file;
   /**
    * An array of COntent Sources linked to this page in {@link $source_file}
    * @var array
    */
   var $sources = array();
   
   /**
    * An array of COntent Sources linked to this page in {@link $source_file}
    * @var array
    */
   var $includes = array();


   var $translations = array();
   
   /**
    * Class constructor method
    * @return void
    */
    function NoAhWebPage()
    {
    }
    
    /**
     * Returns an array of all properties of this class
     * @return array
     */
    function getPageDetails()
    {
       $det = array();
       foreach( $this as $k=>$v )
       {
           $det[$k] = $v;
       }
       return $det;
    }
    
    /**
     * Adds a name source to {@link $sources}
     * @param string $name name of the source
     * @param string $source name of the content source
     * @return void
     */
    function addSource( $name, $source ){
       $i = count($this->sources);
       $this->sources[$i]['name'] = $name;
       $this->sources[$i]['source'] = $source;
    }

    function addTranslation( $name, $langfile )
    {
        $i = count($this->translations);
        $this->translations[$i]['name'] = $name;
        $this->translations[$i]['langfile'] = $langfile;
    }

    /**
     * Looks for a content source metafile linked to this page.
     * 
     * Function use {@link $name} and looks for a file called
     * page_[name].xml This file will contain information on any 
     * content sources which are linked to this page.
     * 
     * This function sets up {@link $source_file}
     * 
     * Returns true if a source file was found, false if not.
     * 
     * @return boolean
     */
    function hasSource()
    {
        $found = false;
        $corefile = CORE_PATH.'/xml/page_'.strtolower($this->name).'.xml';
        $localfile = XML_PATH.'/page_'.strtolower($this->name).'.xml';
        
        error_log("CORE SOURCE : $corefile");
        error_log("LOCAL SOURCE : $localfile");
        
        if ( is_file( $corefile) ) {
            error_log("FOUND CORE");
            $this->source_file = $corefile;
        } else if (is_file( $localfile ) ) {
            error_log("FOUND LOCAL");
            $this->source_file = $localfile;
        }
        if ( is_file($this->source_file ) ) {
            $found = true;
        }
        
        return $found;
    }
         
    /**
     * Loads {@link $source_file} XML file and adds source details to {@link $sources} 
     * 
     * Returns true if sources were actually found in the source file and false if not.
     * 
     * @return boolean
     */
    function findSources()
    {
        $found = false;
        if ( ! $this->hasSource() ) return $found;
        
        if ( is_file($this->source_file ) ) {
            $nrap = new NoAhRap( $this->source_file );
            $xdata = $nrap->getData();

            foreach($xdata as $uri=>$data){
                // look for source namespace.
                if ( substr($uri,0,7)=='source/') {
                    $this->sources[] = $data;
                    $found = true;
                }
                if ( substr($uri,0,8)=='include/') {
                    $this->includes[] = $data;
                    $found = true;
                }
                if ( substr($uri,0,12)=='translation/') {
                    $this->translations[] = $data;
                    $found = true;
                }
            }
        }
        return $found;
    }

}


?>
