<?php
// $Id: xulelement.php,v 1.6 2005/03/22 01:57:22 kerkness Exp $
 /**
  * This file is part of NoAh.
  * 
  * NoAh is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  * 
  * NoAh is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  * 
  * You should have received a copy of the GNU General Public License
  * along with NoAh; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
  
/**
 * Class file for xulelemnt class
 *
 * @author Ryan Mayberry < phparch at gmail dot com >
 * @package xulform
 */

/**
 * Contains methods and variables common to all XUL elements
 * @package xulform
 */
  class xulelement
  {
      /**
       * id / name of our element
       * @var   string      
       */
      var $id;

      /**
       * Friendly Label Value or Language Tag
       * @var   string     
       */
      var $label;

      /**
       * The Value of our xulelement
       * @var   string     
       */
      var $value;

      /**
       * An array of CSS values for element
       * @var   array      
       */
      var $style;

      /**
       * An array of Commands this element might execute
       * @var   array      
       */
      var $commands=array();

      /**
       * An array of Options this element might need as is the case with Menulists and Radiogroups
       * @var   array      
       */
      var $options;
	  
      /**
       * An array of attributes or properties 
       * @var   array      
       */
      var $props=array();
	  
	  
      function xulelement()
      {
      }

      /**
       * Accepts a CSS key/value value combination
       * Eg:
       * setStyle( 'width', 150 );
       * setStyle( 'display', 'none' );
       */
      function setStyle( $key, $value )
      {
          $this->style[$key] = $value;
      }
      
      /**
       * Accepts a CSS String
       * and parses it into array of key value pairs
       */
      function setStyleString( $style )
      {
          // Break apart string to individual css elements
          $cssarr = explode(';',$style);
          // Loop over each CSS element
          foreach($cssarr as $i=>$css)
          {
             // Break apart into Key/Value pair
             $ele = explode(':',$css);
             $this->setStyle($ele[0],$ele[1]);
          }

      }

     /**
      * Returns the css style string for this element
      */
      function getStyle()
      {
          $str = '';
          $i = 0;
          foreach($this->style as $key=>$value)
          {
              if ( $i > 0 ) $str .= ";";
              $str .= "$key:$value";
          $i++;
          }
        return $str;
      }
    
      /**
       * Sets a command for this XUL element to respond to
       * Eg:
       * setCommand('onclick', 'jsFunction()');
       * * Note: any command you include should corrispond with 
       * a javascript that is included in your template.
       */
      function setCommand( $oncall, $function )
      {
          // Check to see if $oncall already has one function defined
          $hascall = false;
          foreach($this->commands as $call=>$func)
          {
              if ( $call == $oncall ) {
                  $hascall = true;
              }
          }
          // If not defined then defined command with blank function
          if ( ! $hascall ) {
              $this->commands[$oncall] = '';
          }

          // Append our function to current functions method
          $this->commands[$oncall] .= $function.' ';
      }

      /**
       * Returns a string of javascript functions to execute from
       * different oncall methods. eg, onclick, oncommand, onfocus ect.
       */
      function getCommandString()
      {
          $str = '';
          foreach( $this->commands as $call=>$func)
          {
              $str .= $call.'="'.$func.'" ';
          }
        return $str;
      }

     /**
      * Adds a single option to the menu
      */
     function addOption($label, $value)
     {
         $this->options[$label] = $value;
     }

     /**
      * Adds a single option to the menu
      */
     function addOptionArray($options)
     {
         foreach($options as $label=>$value)
         {
             $this->addOption($label,$value);
         }
     }
     
    /**
     * 
     */
    function setProperty($name,$value)
    {
         $this->props[$name] = $value;
    }


    /**
     * 
     */
    function getPropertyString()
    {  
       $str = '';
       foreach($this->props as $name=>$value)
       {
         $str .= "$name='$value' ";
       }
      return $str;  
    }


  }
  
?>