<?php
// $Id: rdfmodel.php,v 1.3 2005/03/13 07:20:15 kerkness Exp $

 /**
  * This file is responsible for handeling calls from
  * the RDF modeler.
  */
 include( RDFAPI_INCLUDE_DIR . 'vocabulary/RDF_C.php');

  class NoAhRdfModel
  {
     /**
      * @var array   $subjects
      */
      var $subjects  =   array();

     /**
      * @var array  $predicates
      */
      var $predicates = array();

     /**
      * @var array  $objects
      */
      var $objects   =   array();

     /**
      * @var array  $statements
      */
      var $statements = array();

     /**
      * @var    object  $_model
      * 
      */
      var $_model;
      var $name;
      var $path;


     /**
      * Constructor
      */
      function NoAhRdfModel( $mem='' )
      {
          // Create an instance of the RAP Model:Factory
          $this->_model =& ModelFactory::getDefaultModel();

          if ( $mem ) {
              $this->name=$mem;
              $this->loadFromMemory();
          }
      }

      /**
       * Function just defines core, xml files for NoAh
       */
      function coreFiles(){
          $core = array(
              'system_mainmenu.xml',
              'system_rdfmodel_template.xml'
              );
          return $core;
      }
      
      /**
       * Loads an XML file into model memory.
       */
      function loadxml( $file_path )
      {
          if ( ! $this->path ) $this->guessFilePath($file_path);

          if ( ! is_file($this->path.$file_path) ) {
              $this->_model->load( CORE_PATH.'/xml/system_rdfmodel_template.xml' ); 
          } else {
              $this->_model->load( $this->path.$file_path ); 
          }

          // $this->_model->load( $this->path.$file_path ); 
          $it = $this->_model->getStatementIterator();
          /**
           */
           while ($it->hasNext()) {
              $statement = $it->next();
              $i = $it->getCurrentPosition();
              $this->subjects[$i] = $this->formatURI($statement->getLabelSubject());
              $this->predicates[$i] = $this->formatURI($statement->getLabelPredicate());
              $this->objects[$i] = $this->formatURI($statement->getLabelObject());
           }

           // Get the name of this model.
           $patharr = explode('/',$file_path);
           $revarr = array_reverse($patharr);
           //foreach( $revarr as $k=>$v ){
             //  error_log("$k = > $v ");
           //}

           $this->name = $revarr[0];

      }

      // Function guesses if the file is in
      function guessFilePath( $file ){
        if ( is_file(CORE_PATH.'/xml/'.$file ) ) {
              $this->path = CORE_PATH.'/xml/';
          } else {
              $this->path = XML_PATH.'/';
        }
      }

      /**
       * Loads a model from _SESSION['mem']
       */
      function loadFromMemory()
      {
          $this->name = $_SESSION['mem']['model_name'];
          $this->subjects = $_SESSION['mem']['sub'];
          $this->predicates = $_SESSION['mem']['prd'];
          $this->objects = $_SESSION['mem']['obj'];
//          error_log('Loaded session memory for '.$this->name);
      }

      function clearModel()
      {
          $this->name = '';
          $this->subjects = array();
          $this->predicates = array();
          $this->objects = array();
      }

      // Save the serialzed view of our model to a file.
      function saveModel()
      {
         
         if ( ! $this->path ) $this->guessFilePath($this->name);

         $filename = $this->path.$this->name;

         // loop over all statements array and submit them
         // to our model object. 
         $this->buildModel();

        if ( ! $this->_model->saveAs($filename, "rdf") ){
             error_log("Could not write $filename");
             return false;
         }
         
         return true;

      }

      // loops over all statements and builds our model fully in objects
    //  function buildModel(){
    //     foreach( $this->subjects as $k=>$subject ){
    //         $this->buildStatement($subject,$this->predicates[$k],$this->objects[$k]);
    //     }
    //  }

      // Build our module from the subjects and predicates we have in memory.
      // 
      function buildModel(){

          // Build an array of unqiue subjects
          $mysubs = array();
          foreach( $this->subjects as $k=>$subject ){
              if ( ! in_array($subject, $mysubs ) ) {
                  $mysubs[] = $subject;
              }
          }

          // For each unqiue subject
          foreach( $mysubs as $i=>$subject ){

              error_log("considering: $subject");

              // create a resource for our subject 
              $sub = $this->normalizeURI($subject);
              if ( substr($sub,0,4)!='RDF:' ) {
               $mySub = new Resource($sub) ;
              } else {
               $sub = str_replace('RDF:','',strtoupper($sub));
               $mySub = RDF::$sub();
              }
              
              error_log("Subject Resource = $sub");

              // Get all the statements for this subject.
              $statements = array();
              foreach($this->subjects as $k=>$v){
                  if ( $v == $subject ) {
                      $statments[] = $k;
                  }
              }

              // Loop over every statement for this subject and add 
              // them to our model.
              foreach($this->subjects as $id=>$v){

                  if ( $v != $subject) {
                      continue;
                  }

                  // Normalize URL namespace for sub and prd
                  $prd = $this->normalizeURI($this->predicates[$id]);
                  $obj = $this->normalizeURI($this->objects[$id]);

                  // Clean the object
                  $obj = $this->cleanObject($obj);

                  error_log("Statement = $sub : $prd : $obj");

                  if ( substr($prd,0,4)!='RDF:' ) {
                   $myPrd = new Resource($prd) ;
                  } else {
                      $prd = str_replace('RDF:','',strtoupper($prd));
                      $myPrd = RDF::$prd();
                  }

                  if ( substr($obj,0,7)=='http://' ) {
                   $myObj = new Resource($obj) ;
                  } else if (substr($obj,0,4)=='RDF:') {
                      $obj = str_replace('RDF:','',strtoupper($obj));
                      $myObj = RDF::$obj();
                  }else{
                   $myObj = new Literal($obj);
                  }

                  $statement = new Statement ($mySub, $myPrd, $myObj);
                  $this->_model->add($statement);

              }  // End each statement

         } // End for each unique subject
      }


      // Update or set the MOdel Name
      function setName( $name )
      {
          $this->name = $name;
      }

      // Check to see if there is already an XML model file
      // with the name $this->name
      function hasFile()
      {

          if ( ! $this->path ) $this->guessFilePath($this->name);
          error_log("looking for ".$this->name );
          $dir = $this->path;
          // Make sure directory exists
          if (is_dir($dir)) {
              // values to avoid in considering file names.
              $avoid_arr = array('.','..');
             // Open the directory
             if ($dh = opendir($dir)) {
                 // for every file in the directory
                 while (($file = readdir($dh)) !== false) {
                     // validate the name and add to xml_files array
                     if ( ! in_array($file,$avoid_arr)) {
                         $xml_files[] = $file;
                     }
                 }
                 closedir($dh); // Close the directory.
             }
          }
          if ( in_array($this->name,$xml_files)) {
              error_log("found ".$this->name );
              return true;
          }
          error_log("didn't find ".$this->name );
          return false;
      }

      // Make a backup copy of our Module File.
      function backupModel( $bakname )
      {
          if ( ! $this->path ) $this->guessFilePath($this->name);
          
          $file = $this->path.$this->name;
          $newfile = $this->path.'bak/'.$bakname;

           if (!copy($file, $newfile)) {
               error_log("failed to copy $file...");
               return false;
           }
           error_log("Copied $file -> $newfile");
         return true;
      }

      /**
       * Updates _SESSION['mem'] with model details.
       */
      function updateMemory()
      {
          $_SESSION['mem']['model_name'] = $this->name;
          $_SESSION['mem']['sub'] = $this->subjects;
          $_SESSION['mem']['prd'] = $this->predicates;
          $_SESSION['mem']['obj'] = $this->objects;
//        error_log('Updating session memory for '.$this->name);
      }

     /**
      * Returns an array of statements.
      */
     function getStatementArr()
     {
         $st = array();
         foreach($this->subjects as $i=>$sub)
         {
             $st[$i]['sub'] = $sub;
             $st[$i]['prd'] = $this->predicates[$i];
             $st[$i]['obj'] = $this->objects[$i];
         }
        return $st;
     }

     // Returns the RDF source for the memmodel
     function serializeXML()
     {
         // Serialize the source of the model
          include(RDFAPI_INCLUDE_DIR . 'syntax/SyntaxRDF.php');
          $ser = new RDFSerializer();
          $rdf =& $ser->serialize($this->_model);

         return $rdf;
     }

     /**
      * 
      */
      // function formates know namespaces to a shorter version of thier URI
      function formatURI( $res )
      {
          // Check if URI or Literal
          if ( substr($res,0,7)=='http://' ) {
             $res = str_replace('http://www.streetfighter.ca/ns#','noah:',$res);
             $res = str_replace('http://www.streetfighter.ca/ns/','../ns/',$res);
             $res = str_replace('http://www.w3.org/1999/02/22-rdf-syntax-ns#','RDF:',$res);
             if (substr($res,0,5)=='RDF:_') {
              $res = "RDF:li";
             }
          }
        return $res;
      } 

      function normalizeURI($res)
      {
          $res = str_replace('noah:','http://www.streetfighter.ca/ns#',$res);
          $res = str_replace('../ns/','http://www.streetfighter.ca/ns/',$res);
          if (substr($res,0,5)=='RDF:_') {
           $res = "RDF:li";
          }
        return $res;
      }

      // This function should clean our object for storage in an xml document
      function cleanObject($obj)
      {
         return $obj;
      }

     /**
      * Method returns values for a resource select list
      */
      function getListItems( $restype ){

          $items = array();
          $defaultitems = array();
          switch ( $restype ) {
          case 'updateSubjectList':
              $baseitems = $this->subjects;
              foreach($this->objects as $k=>$v){
                  if (substr($v,0,6)=='../ns/' || 
                      substr($v,0,7)=='http://') {
                      $baseitems[] = $v;
                  }
              }
              break;
          case 'updatePredicateList':
              $baseitems = $this->predicates;
              $defaultitems[]="RDF:type";
              $defaultitems[]="RDF:li";
              $defaultitems[]="RDF:Blank";
              break;
          case 'updateObjectList':
              $baseitems = $this->objects;
              $defaultitems[]="RDF:Seq";
              $defaultitems[]="RDF:Blank";
              break;
          }

          // Loop over items and remove duplicates.
          if ( $baseitems ) {
          foreach($baseitems as $k=>$v){
              if ( ! in_array($v, $items) ) {
                  $items[] = $v;
              }
          }
          }

          if ( $defaultitems ) {
              foreach($defaultitems as $k=>$v){
                  if ( ! in_array($v, $items) ) {
                      $items[] = $v;
                  }
              }
          }

          sort($items);

          return $items;
      }

     /**
      * Searches for a specific statement matching on all sub, prd and obj
      * or just on sub, prd.  When matching on just sub, prd consider that
      * you will only get the first matching index in return.
      * @var	
      */
      function isStatement( $sub, $prd, $obj='' ){
          // Check to see if statement exsists and return index.

          foreach($this->subjects as $i=>$subject ){

              //echo("DOES $subject == $sub ? <br/>");

              if ( $subject != $sub ) continue;
              
              //echo("YES DOES $subject == $sub DOES <br/>");
              //error_log("$subject == $sub");
              //error_log($this->predicates[$i]." == $prd");
              
            if ( $obj ) {  
              if ( $this->predicates[$i] == $prd && 
                   $this->objects[$i] == $obj ) {
                  return $i;
              }
            }else{   // NO OBJECT PROVIDED. 
              //echo("DOES {$this->predicates[$i]} == $prd ?? <br/>");
              if ( $this->predicates[$i] == $prd ) {
              //echo("YES YES YES YES YES $i DOES<br/>");
                  return $i;
              }
            }
          }
          return false;
      }

      function addStatement( $sub, $prd, $obj ){
          // Submit a new statement. 
          $this->subjects[] = $sub;
          $this->predicates[] = $prd;
          $this->objects[] = $obj;
      }
      
      function updateStatement( $id, $sub, $prd, $obj ){
         // Update a statement at index $id. 
         $this->subjects[$id] = $sub;
         $this->predicates[$id] = $prd;
         $this->objects[$id] = $obj;
      }

      function deleteStatement( $id ){
         unset($this->subjects[$id]);
         unset($this->predicates[$id]);
         unset($this->objects[$id]);
      }

      function getStatementAt($id)
      {
          $s['sub'] = $this->subjects[$id];
          $s['prd'] = $this->predicates[$id];
          $s['obj'] = $this->objects[$id];
          return $s;
      }
      
      function addStatementArray( $arr ){
        foreach( $arr as $i=>$data ){
	        $this->addStatement( $data['sub'], $data['prd'], $data['obj'] );
        }
//        error_log('Loaded session memory for '.$this->name);
      }
      


  }



?>
