<?php

// $Id: save_source.php,v 1.3 2005/03/22 02:01:21 kerkness Exp $

// Include our rdfmodeler class file.
require_once( MOD_PATH.'/class/rdfmodel.php' );

/**
 * This file supports the saving of a source from from both
 * the Content Source manager and Database Wizard.
 * 
 * Both these store source details in the _SESSION['mem'] 
 * array, but under different names.
 * 
 * csources and cpieces for the content source manager
 * dsources and dpieces for the database wizard.
 * 
 * The first step is to figure out which _SESSION info we should
 * user to save our source file.
 * 
 * To accomplish this I will default to the Content Source session
 * and set a flag in dbwizard_index if I want to override and use 
 * the dsource sesssion info.
 * 
 */
 
 if ( PKG_NAME == 'dbwizard' ){
   $source_sess = 'dsource';
   $pieces_sess = 'dpieces';
  } else if ( PKG_NAME == 'contentsources'){
   $source_sess = 'csource';
   $pieces_sess = 'cpieces';
  }
 
 
 // Pull our data out of the session.
 $source = $_SESSION['mem'][$source_sess];
 $pieces = $_SESSION['mem'][$pieces_sess];

/**
 * Start save source process.
 * 
 * 
 * 
 *
 */

$mymodel = new NoAhRdfModel();

// DEFINE STATEMENTS FOR SOURCE HEADER INFROMATION

// Set our Source URL
$uri = '../ns/source/' . str_replace( ' ','',$source['label']);
$mymodel->addStatement($uri,'RDF:type','noah:source');

// label
$mymodel->addStatement($uri,'noah:label',$source['label']);
// name
$mymodel->addStatement($uri,'noah:name',$source['name']);
// table
$mymodel->addStatement($uri,'noah:table',$source['table']);
// prime
$mymodel->addStatement($uri,'noah:prime',$source['prime']);
// allowedit
$mymodel->addStatement($uri,'noah:allowedit',$source['allowedit']);
// type
$mymodel->addStatement($uri,'noah:type',$source['type']);
// dbconn
$mymodel->addStatement($uri,'noah:dbconn',$source['dbconn']);
// access
$mymodel->addStatement($uri,'noah:access',$source['access']);

// kill our URI to prevent accidentally using it again
unset($uri);

// DEFINE STATEMENTS FOR EACH CONTENT PIECE
$puri = '../ns/pieces/' . str_replace( ' ','',$source['label']);
$mymodel->addStatement($puri,'RDF:type','RDF:Seq');
$prime_added = false;
foreach($pieces as $i =>$piece )
{   
   $piece['label'] = ( isset($piece['label']) ) ?$piece['label']:$piece['name'];

   if ( $piece['name'] == $source['prime'] ) {
       $prime_added = true;
   }
   
   // Set the url for our piece.
   $uri = '../ns/piece/'. str_replace(' ','', $piece['label']);
   
   // Add to our pieces sequence
   $mymodel->addStatement($puri,'RDF:li',$uri);
   
   // Add statements specific for this piece.
   $mymodel->addStatement($uri,'noah:label',$piece['label']);
   if (isset($piece['input'])) $mymodel->addStatement($uri,'noah:input',$piece['input']);
   $mymodel->addStatement($uri,'noah:name',$piece['name']);
   if (isset($piece['required'])) $mymodel->addStatement($uri,'noah:required',$piece['required']);
   if (isset($piece['order'])) $mymodel->addStatement($uri,'noah:order',$piece['order']);
   if (isset($piece['length'])) $mymodel->addStatement($uri,'noah:length',$piece['length']);
   if (isset($piece['default'])) $mymodel->addStatement($uri,'noah:default',$piece['default']);
   
   // Check piece for joins and build statements for piece joins
   if ( isset($piece['joins']) && $piece['joins'] ){
   	   $pjuri = '../ns/joins/'. str_replace(' ','', $piece['label']);
   	   // add main join statemtnts block to piece
   	   $mymodel->addStatement($uri,'noah:join',$pjuri);
   	   $mymodel->addStatement($pjuri,'RDF:type','RDF:Seq');
   	   // Loop over each join and add statements
   	   foreach($piece['joins'] as $ji=>$join){
   	      $juri = '../ns/join/'. str_replace(' ','', $piece['label']).$join['jfield'];
   	      $mymodel->addStatement($pjuri,'RDF:li',$juri);
		   if (isset($join['jlist'])) $mymodel->addStatement($juri,'noah:jlist',$join['jlist']);
  	 	   if (isset($join['jsource'])) $mymodel->addStatement($juri,'noah:jsource',$join['jsource']);
 	 	   if (isset($join['jfield'])) $mymodel->addStatement($juri,'noah:jfield',$join['jfield']);
		   if (isset($join['jdisplay'])) $mymodel->addStatement($juri,'noah:jdisplay',$join['jdisplay']);   	      
		  unset($juri); 
   	   }
   	   unset($pjuri);
    }
   
   // kill our URI to prevent accidentally using it again
   unset($uri);

}

// Check to see if PRIME KEY field is already a content piece. If it 
// isn't then we will add it here. This should primarily trigger only
// when we are saving brand new content sources.
if ( ! $prime_added ) {
    // Set the url for our piece.
    $uri = '../ns/piece/'. str_replace(' ','', $source['prime']);
    // Add to our pieces sequence
    $mymodel->addStatement($puri,'RDF:li',$uri);
    // Add statements specific for this piece.
    $mymodel->addStatement($uri,'noah:label',$source['prime']);
    $mymodel->addStatement($uri,'noah:input','hidden');
    $mymodel->addStatement($uri,'noah:name',$source['prime']);
    $mymodel->addStatement($uri,'noah:required',1);
    $mymodel->addStatement($uri,'noah:order',1);
    $mymodel->addStatement($uri,'noah:length','11');
    unset($uri);
}

// SET DEFAULT STATEMENTS WHICH NEED TO BE INCLUDED IN ALL SOURCES

// NOTE: This should change. Worried about exopsing database connection
// details too easily if stored in an xml document. Change. Change Change !!
$mymodel->addStatement('../ns/db/Default','noah:xml','connections.xml');


// Set the name of our model
$mymodel->setName($source['name']);

// Save our model
if ( ! $mymodel->saveModel() ){
     $results = "Failed to Save Content Source File.";
} else {
     $results = "success";
}

$noAh->addContent('results',$results);


?>