<?php
/**
 * NoAh Installer goes through the process of installing
 * and configuring NoAh to run for the first time.
 * 
 * At the end of this process the User should have a functioning
 * default NoAh configuration and web site.
 *
 * Start the process by setting the current stage of the install
 * and attempt to run that stage. Each stage checks it's own
 * authorization setup.
 */
if (! session_id() )
{
  session_name('NoAhSession');
  session_start();
}
//phpinfo();
//die();

// Set the current stage.
$stage = ( isset($_GET['stage']) ) ? $_GET['stage'] : 1 ;
$self = $_SERVER['SCRIPT_NAME'];
$selfarr = explode('/',$self);
$self_file = $selfarr[count($selfarr)-1];
$base_path = str_replace( $self_file,'',$_SERVER['SCRIPT_FILENAME'] );

// Create an instance of our update class..
$nUpdate = new NoAhUpdate($stage);

if ( ! $nUpdate->confirmValidStageRequest() ) {
    trigger_error("NoAh failed to validate stage ' $stage ' request ", E_USER_ERROR );
}
// Reset stage to currently valid one
$stage = $nUpdate->getStage();

//echo("current stage is $stage <br/>");

/**
 * Stage 1: Check for required folders, and confirm these folders
 * have proper permissions for the installation process
 * 
 * Critical to only run by non priveledge people if there 
 * is no noah_conf file with complete settings. 
 *******************************************************/
if ( $stage === 1 ) {

    if ( defined('IS_NEW') && ! defined('CORE_PATH') ) {
        // This must be a new install or 
        $nUpdate->is_new = true;
    } else {
        include(CORE_PATH.'/class/noahlib/NoAhUser.php');
        $noUser =& new NoAhUser();
        if ( ! isset($noUser) || ! $noUser->isAdmin() ) {
            header("Location:".SITE_URL);
            trigger_error("noEh: Permission Denied", E_USER_ERROR );
        }

    }
    // Set the paths in our object
    if ( ! defined('CORE_PATH') ) {
        $nUpdate->core_path = $base_path.'noah';
        $nUpdate->site_path = substr($base_path,0,-1);
        $nUpdate->xml_path = $base_path.'xml';
        $nUpdate->upload_path = $base_path.'uploads';
    } else {
        $nUpdate->loadNoAhConf();
    }

    // Check to see if we passed stage 1
    $perms = $nUpdate->checkPermissions();
    
    if ( $perms['pass'] ) {
        $nUpdate->passStage(1);
    }

    // DISPLAY STAGE ONE TEMPLATE
    include( $nUpdate->core_path.'/modules/noupdate/templates/permission_check.php' );
    $nUpdate->saveMemory();

} // ************ END STAGE 1


// ******************************************** SET UP CORE DEFINITIONS 
// As required for further stages
if ($stage > 1 && $stage <= 3 ) {
  if ( ! defined('CORE_PATH') ) {
    define('CORE_PATH', $nUpdate->core_path );
    define('XML_PATH', $nUpdate->xml_path );
  }
}

//
if ($stage > 3) {
  if ( ! defined('CORE_PATH') ) {
    include( $nUpdate->site_path.'/noah_conf.php' );
  }
}

if ( $stage > 1) {
include_once( CORE_PATH . '/class/noahlib/NoAhObject.php' );
include_once( CORE_PATH . '/class/noahlib/NoAhRap.php' );
include_once( CORE_PATH . '/class/noahlib/NoAhRequest.php' );
include_once( CORE_PATH . '/class/noahlib/NoAhContentSource.php' );
include_once( CORE_PATH . '/class/noahlib/NoAhContentPiece.php' );
include_once( CORE_PATH . '/class/noahlib/NoAhContentReader.php' );
include_once( CORE_PATH . '/class/noahlib/NoAhUser.php' );
include_once( CORE_PATH . '/class/db/database_mysql.php' );

// Include RAP (PHP-RDF) API FILES
define("RDFAPI_INCLUDE_DIR", CORE_PATH."/class/rdfapi-php/api/");
include_once(RDFAPI_INCLUDE_DIR . "RdfAPI.php");
include_once( CORE_PATH.'/modules/nosystem/class/rdfmodel.php');
}


/**
 * Stage 2: Set the properties for noah_conf file. This should 
 * include the default database settings.  Save these
 * settings to noah_conf
 *******************************************************/
if ( $stage == 2 ) {

    include(CORE_PATH.'/modules/noupdate/scripts/build_conf_form.php');    

} // ************ END STAGE 2
/**
 * Stage 3: Select which NoAh Modules to install. NoSystem, NoUser
 * & NoPages are required if installing for the first time
 * otherwise it should be optional to update them.
 *******************************************************/
if ( $stage == 3 ) {

// Take an inventory of installed files.
// And what's available
  $nUpdate->takeInventory();
  $modules = $nUpdate->getAvailable();
  $installed = $nUpdate->getInstalled();
  ksort($modules);

// Handle submission of update settings
 if ( isset($_POST['mod_token']) && isset($_SESSION['mod_token']) && $_POST['mod_token']==$_SESSION['mod_token'] ) {
   // pass submission onto new stage
   foreach( $modules as $mod => $pkgs ) {
      $nUpdate->addModule($mod,$_POST['option_'.$mod]);
   }
   unset($_SESSION['mod_token']);
   $nUpdate->passStage(4);
   $nUpdate->saveMemory();
   header("location: ".$self."?stage=5");
 }
  

  $mod_token = md5(rand(10000,99999).'_'.time());
  $_SESSION['mod_token'] = $mod_token;
  // Required Modes
  $required = array('nosystem','nousers','nopages','noupdate');

  include(CORE_PATH.'/modules/noupdate/templates/select_modules_basic.php');    
    
  // Provide a basic form for updating modules
  
  // include(CORE_PATH.'/modules/noupdate/scripts/build_conf_form.php');    

} // ************ END STAGE 3
/**
 * Stage 4: Advance Mode: Load an advance update option for any 
 * module. This option will load an array of potential
 * properties that this module manages. This will be built
 * from all pkg/page/source xml files. Basically all resources
 * and their properties. Update / Don't Update for each one.
 *******************************************************/


/**
 * Stage 5: Copy all pkg/page/source files to the domain folder
 * and install base database tables and content.
 *******************************************************/
if ( $stage == 5 ) {

   installer_header();
   ?>
   <h2>Installing NoAh...</h2>

   <p>NoAh is now installing, you're patience is appreciated.</p>

   <?php flush(); 
   
   $n=0;
   foreach( $nUpdate->modules as $i=>$mod )
   {
       $mod->loadDefaults( $nUpdate->available[$mod->module] );
       echo("Installing module '".$mod->module."' <br/>");

       $mod->processUpdate();

       flush();
       //print_r($mod);
       sleep(1);
   $n++;
   //if ( $n == 1 ) break;
   }
   flush();
   ?>
   <h2>Indexing Pages</h2>
   <?php
   include(CORE_PATH.'/modules/noupdate/scripts/index_pages.php');    
   flush();
   ?>
   <h2>Indexing Modules and Packages</h2>
   <?php
   include(CORE_PATH.'/modules/noupdate/scripts/index_modules.php');    
   flush();
   ?>
   <h2>Indexing Content Sources</h2>
   <?php
   include(CORE_PATH.'/modules/noupdate/scripts/index_sources.php');    
   flush();
   ?>
   <h2>Indexing Translation Files</h2>
   <?php
   include(CORE_PATH.'/modules/noupdate/scripts/index_translations.php');    
   flush();
   
   if ( $nUpdate->is_new ) {
   ?>
   <h2>Running Install SQL</h2>
   <?php
   include(CORE_PATH.'/modules/noupdate/scripts/install_sql.php');    
   flush();
   }
   // Dynamically determine the 'next stage' before setting
   // is_new to  0
   $next_stage = 7;
   if ( $nUpdate->is_new ) {
       $next_stage = 6;
   }
   ?>

  <p><b>Done.</b> Click the "Complete NoAh Setup" button to proceed.</p>
  <table class="outer" width="100%">
      <tr><td class="head" align="right">
       <input type="button" value="Complete NoAh Setup" 
          onclick="javascript: window.location = '<?php echo $self ?>?stage=<?php echo $next_stage ?>'"/>   
          </td></tr>
  </table>

<?php   installer_footer(); 

 // Complete this stage. Clear 'new' flag as we don't
 // want to run any 'install sql' after this stage.
 $nUpdate->is_new = 0;
 $nUpdate->passStage($next_stage-1);
 $nUpdate->saveMemory();


} //******************  END STAGE 5
 /**
  * Stage 6: * Confirm That we have an administrative user
  * everything is ready to run NoAh.
  * Set NoAh Administrative username and password  
  ********************************************************/
if ( $stage == 6 ) {

   if ( isset($_POST['admin_token']) && isset($_SESSION['admin_token']) && $_POST['admin_token']==$_SESSION['admin_token'] ) {
       unset($_SESSION['admin_token']);

     // Make sure the admin username and password does not contain bad data
     // Im not bothering with verbose validation here. If the admin user wants to bugger
     // up their registration their are welcome..   perhaps later this will need impoved 
     // anyway
     $user = strip_tags($_POST['admin_user']);
     $user = str_replace(' ','',trim($user));
     $pass = md5(trim($_POST['admin_pass']));

     // Register user and add them as admin
     $mydb = new NoAhDatabase();
     $mydb->openConnection();

     $sql = "INSERT INTO nousers SET username='".$user."', password='".$pass."', email='".$_POST['admin_email']."'";
     if ( ! $mydb->query($sql) ) {
         trigger_error( $mydb->error()."<br/>$sql<br/>");
     }

     // get admin id
     $adminid = $mydb->insertId();

     $sql = "INSERT INTO nogroup_users SET gid=1, uid=$adminid";
     if ( ! $mydb->query($sql) ) {
         trigger_error( $mydb->error()."<br/>$sql<br/>");
     }
     $sql = "INSERT INTO nogroup_users SET gid=2, uid=$adminid";
     if ( ! $mydb->query($sql) ) {
         trigger_error( $mydb->error()."<br/>$sql<br/>");
     }

     $nUpdate->passStage(6);
     $nUpdate->saveMemory();

     header("Location: ".$self."?stage=7");

   }
   include_once(CORE_PATH.'/class/noahlib/NoAhFormBuilder.php');
   $aForm = new NoAhFormBuilder('aform');

   $aForm->addFormField('admin_user','User Name','text', '' );
   $aForm->addFormField('admin_pass','Password','password', '' );
   $aForm->addFormField('admin_pass_conf','Confirm','password', '' );
   $aForm->addFormField('admin_email','Email','text', '' );

   // Create a token
   $_SESSION['admin_token'] = md5(rand(10000,99999).'_'.time());
   $aForm->addFormField('admin_token','','hidden',$_SESSION['admin_token']);


   $fields = $aForm->renderFields();
   include( CORE_PATH.'/modules/noupdate/templates/setup_administrator.php' );

}
/**
 * Stage 7: Clean up file permissions. Ensure noah_conf.php is 
 * protected. 
 ********************************************************/
if ( $stage == 7 ) {

    // Kill the update object and _session memory.
    unset($_SESSION['mem']['updater']);
    unset($nUpdate);

    // Check to see if our noah_conf file is protected
    $conf_safe = false;
    if ( ! is_writable(SITE_PATH.'/noah_conf.php') ) {
        $conf_safe = true;
    }

    installer_header();
   ?>

  <h2>NoAh Installation is Complete.</h2>
  <p>To test your NoAh
  site just visit your homepage</p>
  <p><a href="<?php echo SITE_URL ?>"><?php echo SITE_URL ?></a></p>

  <p>
  For support information please visit :<br/>
  <a href="http://phparch.sourceforge.net">phparch.sourceforge.net</a>
  
  </p>

  <?php if ( ! $conf_safe ) { ?>
  <p><b><span style="color:red">Notice:</span></b> You're 'noah_conf.php' file 
  is still writeable. You should change the permissions on this file to that it is
  only readable ( chmod 444 )</p>
  <?php } ?>

  <table class="outer" width="100%">
      <tr><td class="head" align="right">
       <input type="button" value="Go to your Homepage" 
          onclick="javascript: window.location = '<?php echo SITE_URL ?>'"/>   
          </td></tr>
  </table>

<?php
    installer_footer();
} // END STAGE 7
/**
 * NoAhUpdate Class
 ********************************************************/
class NoAhUpdate
{
    /**
     * flag to denote if this is a new install or an update
     * @var     boolean
     */
    var $is_new = false;
    /**
     * An array of {@link NoAhModuleUpdate} objects which
     * each contrain the properties of a single modules update setting 
     * @var     array
     */
    var $modules = array();
    /**
     * An array of paths for the noah_conf file
     * @var     array
     */
    var $core_path;
    var $site_path;
    var $xml_path;
    var $upload_path;
    var $default_host;
    var $default_port;
    var $default_user;
    var $default_pass;
    var $default_database;
    var $site_url;

    var $available = array();
    var $installed = array();
    
    /**
     * Time stamp for when this installation process started
     * @var     interger
     */
    var $start_time; 
    /**
     * Time stamp for when this installation process ends
     * @var     interger
     */
    var $end_time; 
    /**
     * Current stage of Installation/Update
     * @var     interger
     */
    var $stage; 
    var $passed = array();

    function NoAhUpdate( $stage )
    {
        // Check for an load _Session memory
        $this->loadMemory();
        // set the current stage stage
        $this->stage = $stage;
    }

    function loadMemory()
    {
        if ( isset( $_SESSION['mem']['updater'] ) ) {
            foreach($_SESSION['mem']['updater'] as $name=>$value )
            {
                $this->$name = $value;
            }
        }
    }

    function saveMemory()
    {
        foreach($this as $name=>$value )
        {
            $_SESSION['mem']['updater'][$name] = $value;
        }
    }

    function confirmValidStageRequest()
    {   
        $valid = false;
        $stage = $this->stage;
        $must_pass = $stage - 1;

        //echo("stage $stage must pass $must_pass <br/>");

        if ( $stage == 1 ) {
            return true;
        }


        if ( ! $this->hasPassed( $must_pass ) ) {
            $this->stage = $must_pass;
            return $this->confirmValidStageRequest();
        }

        return true;
    }

    function hasPassed( $stage )
    {
        if ( isset($this->passed[$stage]) ) {
            return $this->passed[$stage];
        }
        return false;
    }

    function passStage( $stage )
    {
        $this->passed[$stage] = 1;
    }
    function failStage( $stage )
    {
        $this->passed[$stage] = 0;
    }

    function getStage()
    {
        return $this->stage;
    }

    function loadNoAhConf()
    {
        $this->core_path = CORE_PATH;
        $this->xml_path = XML_PATH;
        $this->upload_path = UPLOAD_PATH;
        $this->site_path = SITE_PATH;
    }

    // check for all base required files and folders
    // to run noah.
    function checkPermissions()
    {
        $check['xml'] = ( is_writable( $this->xml_path ) )? 1 : 0 ;
        $check['core'] = ( is_file( $this->core_path.'/handleRequest.php' ) )? 1 : 0 ;
        $check['uploads'] = ( is_writable( $this->upload_path ) )? 1 : 0 ;
        $check['site'] = ( is_file( $this->site_path.'/request.php' ) )? 1 : 0 ;
        
        if ( $this->is_new ) {
            $check['conf'] = ( is_writable( $this->site_path.'/noah_conf.php' ) )? 1 : 0 ;
        }

       // Check to see if all passed
       $pass = 1;
       foreach( $check as $i=>$v )
       {
          if ( ! $v ) $pass = 0;
       }

       $check['pass'] = $pass;
      
       return $check;

    }

    /**
     * This function should take an inventory of what we have installed
     * and what we don't have installed.
     */
    function takeInventory()
    {
        $this->installed = array();
        // Open a known directory, and proceed to read its contents
        if (is_dir($this->xml_path)) {
           if ($dh = opendir($this->xml_path)) {
               while (($file = readdir($dh)) !== false) {
                  if ( strpos($file,'.xml') ) {
                      $this->installed[] = $file;
                  }
               }
               closedir($dh);
           }
        }

        // Get an array of all modules / packages / pages
        $this->findAvailable();
    }

    function findAvailable()
    {
        // Open a known directory, and proceed to read its contents
        $banned = array('CVS','.','..');
        if (is_dir($this->core_path.'/modules')) {
           if ($dh = opendir($this->core_path.'/modules')) {
               while (($file = readdir($dh)) !== false) {
                  if ( ! in_array($file, $banned) ){
                      $this->available[$file] = array();
                  }
               }
               closedir($dh);
           }
        }

        foreach($this->available as $mod=>$pkgs)
        {
            $banned = array('CVS','.','..','bak');
            if (is_dir($this->core_path.'/modules/'.$mod.'/pkgs')) {
               if ($dh = opendir($this->core_path.'/modules/'.$mod.'/pkgs')) {
                   while (($file = readdir($dh)) !== false) {
                      if ( ! in_array($file, $banned)  ) {
                          $this->available[$mod][] = $file;
                      }
                   }
                   closedir($dh);
               }
            }
        }
    }

    function getInstalled()
    {
        return $this->installed;
    }
    function getAvailable()
    {
        return $this->available;
    }


    function addModule( $mod, $option )
    {
       $this->modules[$mod] = new NoAhModuleUpdate($mod,$option);
       $this->modules[$mod]->core_path = $this->core_path;
       $this->modules[$mod]->xml_path = $this->xml_path;
    }

}

/**
 * NoAhModuleUpdate Class
 ********************************************************/
class NoAhModuleUpdate
{
    var $module;
    var $option;
    var $defaults;
    var $core_path;
    var $xml_path;
    var $sync = array();
    var $saved = array();

    /**
     * Class constructor, sets {@link $stage}
     * 
     */
    function NoAhModuleUpdate( $mod, $option )
    {
        $this->module = $mod;
        $this->option = $option;
    }

    function processUpdate()
    {
        foreach( $this->defaults as $i=>$pkg )
        {
            if ( substr($pkg,-4 )!='.xml' ) continue;
            //echo(" - creating syncObject for $pkg {$this->module} {$this->core_path} {$this->xml_path}<br/>");
            $this->sync[$pkg] = new NoAhRdfSync( $pkg, $this->module, $this->core_path, $this->xml_path );
            flush();
            //echo(" - compare statements for $pkg using {$this->option} <br/>");
            $this->sync[$pkg]->compareStatements( $this->option );
            $this->saved[$pkg] = $this->sync[$pkg]->saveXModel();
        }
    }

    function loadDefaults( $available )
    {
        $this->defaults = $available;
    }

}

/**
 * This class will sync two RDF models
 */

class NoAhRdfSync
{
    var $xmodel = array();
    var $nmodel = array();
    var $dmodel = array();
    var $xml_path;
    var $core_path;
    var $module;
    var $file;

    function NoAhRdfSync( $file, $module, $core_path, $xml_path )
    {
        $this->xml_path = $xml_path;
        $this->core_path = $core_path;
        $this->module = $module;
        $this->file = $file;

        $this->dmodel = new NoAhRdfModel();
        $this->dmodel->path = $this->core_path.'/modules/'.$this->module.'/pkgs/';
        $this->xmodel = new NoAhRdfModel();
        $this->xmodel->path = $this->xml_path.'/';

        $this->nmodel = new NoAhRdfModel();
        $this->nmodel->path = $this->xml_path.'/';

        $this->dmodel->loadxml( $this->file );

        if ( is_file( $this->xml_path.'/'.$this->file ) ) {
            //echo("Loading exsisting {$this->file} <Br/>");
            $this->xmodel->loadxml( $this->file );
        } else {
            //echo("Did not file exsisting {$this->file} <Br/>");
        }

    }

    function compareStatements( $option )
    {
        // get an array of statements from our default document 
        $statements = $this->dmodel->getStatementArr();

        //echo("Getting statements from {$this->file} <Br/><br/>");

        // Loop over all statements and compare with xmodel
        foreach($statements as $i=>$s )
        {
           // if list match statement on all three properties
           if ( $s['prd'] == 'RDF:li' ) {
               //echo("Looking for {$s['sub']} {$s['prd']} {$s['obj']}  <Br/>");
               $si = $this->xmodel->isStatement( $s['sub'], $s['prd'], $s['obj'] );
           } else {
               //echo("Looking for {$s['sub']} {$s['prd']} <br/>");
               $si = $this->xmodel->isStatement( $s['sub'], $s['prd'] );
           }

           // If our statement exsists in xml document.
           if ( $si && ! $option == 'all' )  {
               // Re-use exsisting statement
               //echo("Add new statement from exsisting model source<br/>");
               $s = $this->xmodel->getStatementAt($si);
               $this->nmodel->addStatement( $s['sub'], $s['prd'], $s['obj'] );
           } else {
               //echo("Add new statement from source<br/>");
               $this->nmodel->addStatement( $s['sub'], $s['prd'], $s['obj'] );
           }
        }
    }


    function saveXModel()
    {
       $this->nmodel->setName( $this->file );
       $this->nmodel->saveModel();
       return $this->nmodel->getStatementArr();
    }

}

function installer_header(){
?>    
<html>
<head>
 <title>NoAh Installer</title>
</head>
<body>
<style>
td { 
    font-family: Verdana, Arial, Helvetica, sans-serif;
    font-size: 10px;
}
.outer {color:#333333;border: 1px solid silver;}
.head {color:#333333; background-color: #c2cdd6; padding: 5px; font-weight: bold;}
.even {color:#000000;background-color: #dee3e7; padding: 5px;}
.odd {color:#000000;background-color: #E9E9E9; padding: 5px;}
.foot {color:#000000;background-color: #c2cdd6; padding: 5px; font-weight: bold;}
tr.even td { 
    font-family: Verdana, Arial, Helvetica, sans-serif;
    font-size: 10px;
    color:#000000;background-color: #dee3e7; padding: 5px;}
tr.odd td {
    font-family: Verdana, Arial, Helvetica, sans-serif;
    font-size: 10px;
    color:#000000;background-color: #E9E9E9; padding: 5px;}
</style>
<table class="outer" width="60%" align="left">
<tr><td>
<?php }

function installer_footer(){
?>
</td></tr>
</table>
</body>
</html>


<?php
//phpinfo();      
}

  function listContents( $path )
  {
    $myfiles = array();
    // First loope over path and file just the files.
    if ($handle = opendir($path)) {
       while (false !== ($file = readdir($handle))) {
           if ($file != "." && $file != ".." && $file != "CVS") {
               $myfiles[] = $file;
           }
       }
       closedir($handle);
    }
    return $myfiles;
  }


?>
